@@ -27,23 +27,29 @@
    This class is just a nice place to store options for the compiler and their defaults.
 */
 
-namespace Falcon {
-
-class HOptions
+class FalconOptions
 {
+   void modalGiven();
+   bool m_modal;
+   bool m_justinfo;
+
 public:
 
-   String input;
-   String output;
-   String load_path;
-   String io_encoding;
-   String source_encoding;
-   String module_language;
-   List preloaded;
-   List directives;
+   Falcon::String input;
+   Falcon::String output;
+   Falcon::String load_path;
+   Falcon::String io_encoding;
+   Falcon::String source_encoding;
+   Falcon::String module_language;
+#ifndef NDEBUG
+   Falcon::String trace_file;
+#endif
+
+   Falcon::List preloaded;
+   Falcon::List directives;
+   Falcon::List defines;
 
    bool compile_only;
-   bool assemble_only;
    bool run_only;
    bool tree_out;
    bool assemble_out;
@@ -51,44 +57,27 @@ public:
    bool force_recomp;
    bool check_memory;
 
-   bool via_asm;
    bool comp_memory;
    bool recompile_on_load;
-
    bool save_modules;
-
    bool wait_after;
-
    bool parse_ftd;
 
    bool compile_tltable;
+   bool interactive;
 
-   HOptions():
-      input( "" ),
-      output( "" ),
-      load_path( "" ),
-      compile_only( false ),
-      check_memory( false ),
-      assemble_only( false ),
-      assemble_out( false ),
-      run_only( false ),
-      tree_out( false ),
-      via_asm( false ),
-      search_path( false ),
-      recompile_on_load( true ),
-      comp_memory( true ),
-      save_modules( true ),
-      force_recomp( false ),
-      io_encoding( "" ),
-      source_encoding( "" ),
-      wait_after( false ),
-      parse_ftd( false ),
-      compile_tltable( false )
-   {}
+   bool ignore_syspath;
+
+   FalconOptions();
+
+   void parse( int argc, char **argv, int &script_pos );
+   void usage( bool deep = false );
+   void version();
+
+   /** Returns true if the parsed options required an immediate exit. */
+   bool wasJustInfo() const { return m_justinfo; }
 };
 
-}
-
 #endif
 
 /* end of options.h */
