@@ -28,6 +28,8 @@
 namespace Falcon
 {
 
+void gen_code( Module *mod, const FuncDef *fd, Stream *out, const t_labelMap &labels );
+
 SymbolTable *global_symtab;
 SymbolTable *current_symtab;
 
@@ -40,9 +42,9 @@ Options::Options():
    m_dump_dep( false ),
    m_isomorphic( false ),
    m_inline_str( false ),
-   m_fname(0),
    m_stdin( false ),
-   m_lineinfo( true )
+   m_lineinfo( true ),
+   m_fname(0)
 {}
 
 Options options;
@@ -93,8 +95,10 @@ void write_string( Stream *output, uint32 stringId, Module *mod )
 
 void write_symbol( Stream *output, const char *prefix, int32 id )
 {
-   if ( options.m_isomorphic ) {
-      if ( *prefix == 'L' || *prefix == 'P' && current_symtab != 0 )
+   if ( options.m_isomorphic ) 
+   {
+      if ( current_symtab != 0 
+           && (*prefix == 'L' || *prefix == 'P')  )
       {
          MapIterator iter = current_symtab->map().begin();
          while( iter.hasCurrent() )
@@ -154,6 +158,7 @@ void write_operand( Stream *output, byte *instruction, int opnum, Module *mod )
          case P_PARAM_LOCID:
          case P_PARAM_PARID:
          case P_PARAM_STRID:
+         case P_PARAM_LBIND:
             offset += 4;
          break;
 
@@ -181,50 +186,51 @@ void write_operand( Stream *output, byte *instruction, int opnum, Module *mod )
                ( opnum == 2 && ( opcode == P_FORK || opcode == P_TRAN ) )
                )
             {
-               write_label( output, endianInt32( *((int32*) (instruction + offset ))) );
+               write_label( output, *((int32*) (instruction + offset )) );
                return;
             }
          }
          temp = "0x";
-         temp.writeNumberHex( endianInt32( *((int32*) (instruction + offset ))) );
+         temp.writeNumberHex( *((int32*) (instruction + offset )) );
          output->writeString( temp );
       break;
 
 
       case P_PARAM_INT32:
-         temp.writeNumber( (int64) endianInt32(*((int32*) (instruction + offset ))));
+         temp.writeNumber( (int64) *((int32*) (instruction + offset )));
          output->writeString( temp );
          break;
 
       case P_PARAM_NOTUSED: return;
 
-      case P_PARAM_STRID: write_string( output, endianInt32(*((int32*) (instruction + offset ))), mod ); break;
+      case P_PARAM_STRID: write_string( output, *((int32*) (instruction + offset )), mod ); break;
+      case P_PARAM_LBIND: output->writeString( "$" + *mod->getString( *((int32*) (instruction + offset )))); break;
+
       case P_PARAM_NUM:
-         temp.writeNumber( endianNum( *((numeric*) (instruction + offset ) ) ));
+         temp.writeNumber( loadNum( instruction + offset ));
          output->writeString(temp );
          break;
       case P_PARAM_NIL: output->writeString( "NIL" ); break;
       case P_PARAM_TRUE: output->writeString( "T" ); break;
       case P_PARAM_FALSE: output->writeString( "F" ); break;
       case P_PARAM_INT64:
-         temp.writeNumber( (int64) endianInt64(*((int64*) (instruction + offset ))) );
+         temp.writeNumber( (int64) loadInt64(instruction + offset) );
          output->writeString(temp );
       break;
       case P_PARAM_GLOBID:
-         write_symbol( output, "G", endianInt32(*((int32*) (instruction + offset ))) );
+         write_symbol( output, "G", *((int32*) (instruction + offset )) );
       break;
       case P_PARAM_LOCID:
-         write_symbol( output, "L", endianInt32(*((int32*) (instruction + offset ))) );
+         write_symbol( output, "L", *((int32*) (instruction + offset )) );
       break;
       case P_PARAM_PARID:
-         write_symbol( output, "P", endianInt32(*((int32*) (instruction + offset ))) );
+         write_symbol( output, "P", *((int32*) (instruction + offset )) );
       break;
 
-      case P_PARAM_NTD64: write_label_long( output, endianInt64(*((int64*) (instruction + offset ))) ); break;
+      case P_PARAM_NTD64: write_label_long( output, loadInt64((instruction + offset )) ); break;
       case P_PARAM_REGA: output->writeString( "A" ); break;
       case P_PARAM_REGB: output->writeString( "B" ); break;
       case P_PARAM_REGS1: output->writeString( "S1" ); break;
-      case P_PARAM_REGS2: output->writeString( "S2" ); break;
       case P_PARAM_REGL1: output->writeString( "L1" ); break;
       case P_PARAM_REGL2: output->writeString( "L2" ); break;
    }
@@ -244,6 +250,7 @@ uint32 calc_next( byte *instruction )
          case P_PARAM_LOCID:
          case P_PARAM_PARID:
          case P_PARAM_STRID:
+         case P_PARAM_LBIND:
             offset += 4;
          break;
 
@@ -262,14 +269,17 @@ uint32 calc_next( byte *instruction )
    Isomorphic symbol generators.
 */
 
-void gen_function( Stream *m_out, const Symbol *func )
+void gen_function( Module *module, const Symbol *func, Stream *m_out, t_labelMap labels )
 {
    m_out->writeString( "; ---------------------------------------------\n" );
    m_out->writeString( "; Function " + func->name() + "\n" );
+   String tempOff;
+   tempOff.writeNumberHex( func->getFuncDef()->basePC() );
+   m_out->writeString( "; Declared at offset 0x" + tempOff + "\n" );
    m_out->writeString( "; ---------------------------------------------\n\n" );
    m_out->writeString( ".funcdef " + func->name() );
-   /*if ( func->symbol()->exported() )
-      *m_out << " export";*/
+   if ( func->exported() )
+      m_out->writeString( " export" );
    m_out->writeString( "\n" );
 
    // generate the local symbol table.
@@ -288,6 +298,9 @@ void gen_function( Stream *m_out, const Symbol *func )
          case Symbol::tlocal:
             m_out->writeString( ".local " + sym->name() + "\n" );
          break;
+
+         default:
+            break;
       }
 
       iter.next();
@@ -300,7 +313,11 @@ void gen_function( Stream *m_out, const Symbol *func )
          m_out->writeString( ".param " + sym->name() + "\n" );
    }
 
-   m_out->writeString( "; ---------------------------------------------\n" );
+   m_out->writeString( "; ---\n" );
+
+   gen_code( module, fd, m_out, labels );
+   m_out->writeString( ".endfunc\n\n" );
+
 }
 
 
@@ -328,6 +345,7 @@ void gen_propdef( Stream *m_out, const VarDef &def )
       case VarDef::t_symbol:
          m_out->writeString( "$" +def.asSymbol()->name() );
          break;
+      default: break;
    }
 }
 
@@ -361,16 +379,6 @@ void gen_class( Stream *m_out, const Symbol *sym )
       const InheritDef *id = (const InheritDef *) it_iter->data();
       const Symbol *parent = id->base();
       m_out->writeString( ".inherit $" + parent->name() );
-      ListElement *param_iter = id->parameters().begin();
-      while( param_iter != 0 )
-      {
-         m_out->writeString( " " );
-         const VarDef *vd = ( const VarDef * ) param_iter->data();
-         gen_propdef( m_out, *vd );
-         param_iter = param_iter->next();
-         if( param_iter != 0 )
-            m_out->writeString( "," );
-      }
       m_out->writeString( "\n" );
       it_iter = it_iter->next();
    }
@@ -400,35 +408,6 @@ void gen_class( Stream *m_out, const Symbol *sym )
       m_out->writeString( ".ctor $" + cd->constructor()->name() + "\n" );
    }
 
-   // write has and hasn't clauses
-   if ( ! cd->has().empty() ) {
-      m_out->writeString( ".has " );
-      ListElement *has_iter = cd->has().begin();
-      while( has_iter != 0 )
-      {
-         const Symbol *hsym = (const Symbol *) has_iter->data();
-         m_out->writeString( "$" + hsym->name() );
-         has_iter = has_iter->next();
-         if ( has_iter != 0 )
-            m_out->writeString( ", " );
-      }
-      m_out->writeString( "\n" );
-   }
-
-   if ( ! cd->hasnt().empty() ) {
-      m_out->writeString( ".hasnt " );
-      ListElement *hasnt_iter = cd->has().begin();
-      while( hasnt_iter != 0 )
-      {
-         const Symbol *hsym = (const Symbol *) hasnt_iter->data();
-         m_out->writeString( "$" + hsym->name() );
-         hasnt_iter = hasnt_iter->next();
-         if ( hasnt_iter != 0 )
-            m_out->writeString( ", " );
-      }
-      m_out->writeString( "\n" );
-   }
-
    m_out->writeString( ".endclass\n" );
    m_out->writeString( "; ---------------------------------------------\n" );
 }
@@ -437,27 +416,23 @@ void gen_class( Stream *m_out, const Symbol *sym )
 /**************************************************
    Symbols
 */
-void disassembler( Module *module, Stream *out, const t_labelMap &labels, const t_funcMap &functions )
+void gen_code( Module *module, const FuncDef *fd, Stream *out, const t_labelMap &labels )
 {
    uint32 iPos =0;
-   byte *code;
+   byte *code = fd->code();
    byte opcode;
    int oldline = -1;
 
-   code = module->code();
-   current_symtab = &module->symbolTable();
-   global_symtab = current_symtab;
+   // get the main code
 
-   while( iPos < module->codeSize() )
+   while( iPos < fd->codeSize() )
    {
       opcode = code[ 0 ];
 
       if ( options.m_isomorphic )
       {
-         if ( iPos == module->entry() )
-            out->writeString( ".entry\n" );
 
-         int l = module->getLineAt( iPos );
+         int l = module->getLineAt( fd->basePC() + iPos );
          if ( l != oldline ) {
             String temp;
             out->writeString( ".line " );
@@ -466,13 +441,6 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
             oldline = l;
          }
 
-         t_funcMap::const_iterator tfunc = functions.find( iPos );
-         if ( tfunc != functions.end() ) {
-            FuncDef *fd = tfunc->second->getFuncDef();
-            current_symtab = &fd->symtab();
-            gen_function( out, tfunc->second );
-         }
-
          t_labelMap::const_iterator target = labels.find( iPos );
          if ( target != labels.end() ) {
             String temp;
@@ -561,14 +529,9 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
          case P_LDV : csOpName = "LDV "; break;
          case P_LDP : csOpName = "LDP "; break;
          case P_TRAN: csOpName = "TRAN"; break;
-         case P_UNPK: csOpName = "UNPK"; break;
-         case P_UNPS: csOpName = "UNPS"; break;
+         case P_LDAS: csOpName = "LDAS"; break;
          // when isomorphic, switch is created through directive
          case P_SWCH: csOpName = options.m_isomorphic ? "" : "SWCH"; break;
-         case P_HAS : csOpName = "HAS "; break;
-         case P_HASN: csOpName = "HASN"; break;
-         case P_GIVE: csOpName = "GIVE"; break;
-         case P_GIVN: csOpName = "GIVN"; break;
          case P_IN  : csOpName = "IN  "; break;
          case P_NOIN: csOpName = "NOIN"; break;
          case P_PROV: csOpName = "PROV"; break;
@@ -587,15 +550,13 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
          case P_TRAV: csOpName = "TRAV"; break;
          case P_INCP: csOpName = "INCP"; break;
          case P_DECP: csOpName = "DECP"; break;
-         case P_PASS: csOpName = "PASS"; break;
-         case P_PSIN: csOpName = "PSIN"; break;
 
          case P_SHR : csOpName = "SHR "; break;
          case P_SHL : csOpName = "SHL "; break;
          case P_SHRS: csOpName = "SHRS"; break;
          case P_SHLS: csOpName = "SHLS"; break;
-         case P_LDVR: csOpName = "LDVR"; break;
-         case P_LDPR: csOpName = "LDPR"; break;
+         case P_CLOS: csOpName = "CLOS"; break;
+         case P_PSHL: csOpName = "PSHL"; break;
          case P_LSB : csOpName = "LSB "; break;
          case P_SELE: csOpName = options.m_isomorphic  ? "": "SELE"; break;
          case P_INDI: csOpName = "INDI"; break;
@@ -603,7 +564,10 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
          case P_TRAC: csOpName = "TRAC"; break;
          case P_WRT : csOpName = "WRT "; break;
          case P_STO : csOpName = "STO "; break;
-
+         case P_FORB: csOpName = "FORB"; break;
+         case P_EVAL: csOpName = "EVAL"; break;
+         case P_OOB : csOpName = "OOB "; break;
+         case P_TRDN: csOpName = "TRDN"; break;
          default:
             csOpName = "????";
       }
@@ -631,6 +595,19 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
          }
       }
 
+      // Line marker in non-ismoprphic mode
+      if ( options.m_lineinfo && ! options.m_isomorphic )
+      {
+         int l = module->getLineAt( fd->basePC() + iPos );
+         if ( l != oldline ) {
+            out->writeString( " ; LINE " );
+            String temp;
+            temp.writeNumber( (int64) l );
+            out->writeString( temp );
+            oldline = l;
+         }
+      }
+
       //special swch handling
       if ( opcode == P_SWCH || opcode == P_SELE )
       {
@@ -649,7 +626,7 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
          iPos += advance;
          code += advance;
 
-         uint64 sw_count = (uint64) endianInt64( *((int64 *) (code - sizeof(int64))));
+         uint64 sw_count = (uint64) loadInt64( code - sizeof(int64) );
 
          uint16 sw_int = (int16) (sw_count >> 48);
          uint16 sw_rng = (int16) (sw_count >> 32);
@@ -657,12 +634,12 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
          uint16 sw_obj = (int16) sw_count;
 
          //write the nil operand
-         if ( *reinterpret_cast<int32 *>(code) != 0xffffffff )
+         if ( *reinterpret_cast<uint32 *>(code) != 0xffffffff )
          {
             if ( ! options.m_isomorphic )
                out->writeString( "\t\t" );
             out->writeString( ".case NIL, " );
-            write_label( out, endianInt32( *reinterpret_cast<int32 *>(code) ) );
+            write_label( out, *reinterpret_cast<int32 *>(code) );
             out->writeString( "\n" );
          }
          iPos += sizeof( int32 );
@@ -675,11 +652,11 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
                out->writeString( "\t\t" );
             out->writeString( ".case " );
             String temp;
-            temp.writeNumber( (int64)  endianInt64( *reinterpret_cast<int64 *>(code) ) );
+            temp.writeNumber( (int64)  loadInt64( code ) );
             out->writeString( temp + ", " );
             code += sizeof( int64 );
             iPos += sizeof( int64 );
-            write_label( out, endianInt32(  *reinterpret_cast<int32 *>( code ) ));
+            write_label( out, *reinterpret_cast<int32 *>( code ) );
             out->writeString( "\n" );
             code += sizeof( int32 );
             iPos += sizeof( int32 );
@@ -694,13 +671,13 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
                out->writeString( "\t\t" );
             out->writeString( ".case " );
             String temp;
-            temp.writeNumber(  (int64) endianInt32( *reinterpret_cast<int32 *>(code) ) );
+            temp.writeNumber(  (int64)  *reinterpret_cast<int32 *>(code) );
             temp += ":";
-            temp.writeNumber(  (int64) endianInt32( *reinterpret_cast<int32 *>(code+ sizeof(int32)) ) );
+            temp.writeNumber(  (int64) *reinterpret_cast<int32 *>(code+ sizeof(int32) ) );
             out->writeString( temp + ", " );
             code += sizeof( int64 );
             iPos += sizeof( int64 );
-            write_label( out, endianInt32( *reinterpret_cast<int32 *>( code ) ) );
+            write_label( out, *reinterpret_cast<int32 *>( code )  );
             out->writeString( "\n" );
             code += sizeof( int32 );
             iPos += sizeof( int32 );
@@ -713,11 +690,11 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
             if ( ! options.m_isomorphic )
                out->writeString( "\t\t" );
             out->writeString( ".case " );
-            write_string( out, endianInt32( *reinterpret_cast<int32 *>(code)), module );
+            write_string( out,  *reinterpret_cast<int32 *>(code), module );
             code += sizeof( int32 );
             iPos += sizeof( int32 );
             out->writeString( ", " );
-            write_label( out, endianInt32( *reinterpret_cast<int32 *>( code ) ) );
+            write_label( out, *reinterpret_cast<int32 *>( code ) );
             out->writeString( "\n" );
             code += sizeof( int32 );
             iPos += sizeof( int32 );
@@ -731,7 +708,7 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
                out->writeString( "\t\t" );
 
             out->writeString( ".case " );
-            int32 symId = endianInt32( *reinterpret_cast<int32 *>(code) );
+            int32 symId =  *reinterpret_cast<int32 *>(code);
             Symbol *sym = module->getSymbol( symId );
             if( sym == 0 )
             {
@@ -744,7 +721,7 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
 
             code += sizeof( int32 );
             iPos += sizeof( int32 );
-            write_label( out, endianInt32( *reinterpret_cast<int32 *>( code ) ) );
+            write_label( out, *reinterpret_cast<int32 *>( code ) );
             out->writeString( "\n" );
             code += sizeof( int32 );
             iPos += sizeof( int32 );
@@ -763,20 +740,8 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
 
       }
 
-      if ( options.m_lineinfo && ! options.m_isomorphic )
-      {
-         int l = module->getLineAt( iPos );
-         if ( l != oldline ) {
-            out->writeString( " ; LINE " );
-            String temp;
-            temp.writeNumber( (int64) l );
-            out->writeString( temp );
-            oldline = l;
-         }
-      }
       out->writeString( "\n" );
    }
-
 }
 
 
@@ -785,13 +750,13 @@ void disassembler( Module *module, Stream *out, const t_labelMap &labels, const
    instructions.
 *********************************************************/
 
-void Analizer( Module *module, t_labelMap &labels  )
+void Analizer( FuncDef *fd, t_labelMap &labels )
 {
    byte *code, *end;
    byte opcode;
 
-   code = module->code();
-   end = code + module->codeSize();
+   code = fd->code();
+   end = code + fd->codeSize();
    while( code < end )
    {
       opcode = code[ 0 ];
@@ -800,11 +765,11 @@ void Analizer( Module *module, t_labelMap &labels  )
       switch( opcode )
       {
          case P_FORK:
-            labels[ endianInt32( *(uint32 *)(code + sizeof(int32)*2 ) ) ] = 0;
+            labels[ *(uint32 *)(code + sizeof(int32)*2 ) ] = 0;
          break;
 
          case P_TRAN:
-            labels[ endianInt32( *(uint32 *)(code + sizeof(int32)*2 ) ) ] = 0;
+            labels[ *(uint32 *)(code + sizeof(int32)*2 ) ] = 0;
          // do not break; fallback
 
          case P_SWCH:
@@ -816,7 +781,7 @@ void Analizer( Module *module, t_labelMap &labels  )
          case P_IFF :
          case P_ONCE:
          case P_TRAV:
-            labels[ endianInt32( *(uint32 *)(code + sizeof(int32)) ) ] = 0;
+            labels[  *(uint32 *)(code + sizeof(int32)) ] = 0;
          break;
       }
 
@@ -828,7 +793,7 @@ void Analizer( Module *module, t_labelMap &labels  )
       {
          //special swch handling
          code += sizeof( int32 ) * 3;  // remove instro and two operands
-         uint64 sw_count = (uint64) endianInt64( *((int64 *) (code)));
+         uint64 sw_count = (uint64) loadInt64(code);
 
          uint16 sw_int = (int16) (sw_count >> 48);
          uint16 sw_rng = (int16) (sw_count >> 32);
@@ -838,7 +803,7 @@ void Analizer( Module *module, t_labelMap &labels  )
          code += sizeof( int64 );
 
          //write the nil operand
-         if ( *reinterpret_cast<int32 *>(code) != 0xffffffff )
+         if ( *reinterpret_cast<uint32 *>(code) != 0xffffffff )
          {
             labels[*reinterpret_cast<int32 *>(code)] = 0;
          }
@@ -920,9 +885,10 @@ void write_strtable( e_tabmode mode , Stream *out, Module *mod )
 }
 
 
-void write_symtable( e_tabmode mode , Stream *out, const SymbolTable *st )
+void write_symtable( e_tabmode mode , Stream *out, Module *mod )
 {
    const Symbol *sym;
+   const SymbolTable *st = &mod->symbolTable();
 
    out->writeString( ";--------------------------------------------\n" );
    out->writeString( "; Symbol table\n" );
@@ -932,6 +898,12 @@ void write_symtable( e_tabmode mode , Stream *out, const SymbolTable *st )
    while( iter.hasCurrent() )
    {
       sym = *(const Symbol **) iter.currentValue();
+      if ( sym->name() == "__main__" )
+      {
+         iter.next();
+         continue;
+      }
+
       String temp;
       switch( mode )
       {
@@ -944,12 +916,20 @@ void write_symtable( e_tabmode mode , Stream *out, const SymbolTable *st )
                case Symbol::tlocal: out->writeString( "LOCAL" ); break;
                case Symbol::tparam: out->writeString( "PARAM" ); break;
                case Symbol::tlocalundef: out->writeString( "LU" ); break;
-               case Symbol::tfunc: out->writeString( "FUNC" ); break;
+               case Symbol::tfunc:
+               {
+                  String temp;
+                  temp.writeNumberHex( sym->getFuncDef()->basePC(), true );
+                  out->writeString( "FUNC(0x" + temp + ")" );
+                  break;
+               }
                case Symbol::textfunc: out->writeString( "EXTFUNC" ); break;
                case Symbol::tclass: out->writeString( "CLASS" ); break;
                case Symbol::tprop: out->writeString( "PROP" ); break;
                case Symbol::tvar: out->writeString( "VDEF" ); break;
                case Symbol::tinst: out->writeString( "INST" ); break;
+               default:
+                  break;
             }
             out->writeString( " " + sym->name() + ": " );
             temp.writeNumber( (int64) sym->id() );
@@ -963,21 +943,73 @@ void write_symtable( e_tabmode mode , Stream *out, const SymbolTable *st )
          break;
 
          case e_mode_table:
+            // see if it's imported
+            if ( sym->type() == Symbol::tundef )
+            {
+               if ( sym->imported() )
+               {
+                  // see if we have an alias from where to import it
+                  uint32 dotpos = sym->name().rfind( "." );
+                  if( dotpos != String::npos )
+                  {
+                     String modname = sym->name().subString( 0, dotpos );
+                     String symname = sym->name().subString( dotpos + 1 );
+
+                     temp =  ".import " + symname + " ";
+                     temp.writeNumber( (int64) sym->declaredAt() );
+
+                     ModuleDepData *depdata = mod->dependencies().findModule( modname );
+                     // We have created the module, the entry must be there.
+                     fassert( depdata != 0 );
+                     if ( depdata->isFile() )
+                     {
+                        if( *depdata->moduleName() == modname )
+                           temp += " \"" + modname+"\"";
+                        else
+                           temp += " \"" + *depdata->moduleName() +"\" "+ modname;
+                     }
+                     else
+                     {
+                        if( *depdata->moduleName() == modname )
+                           temp += " " + modname;
+                        else
+                           temp += " " + *depdata->moduleName() +" "+ modname;
+                     }
+                  }
+                  else {
+                     temp =  ".import " + sym->name() + " ";
+                     temp.writeNumber( (int64) sym->declaredAt() );
+                  }
+               }
+               else {
+                  temp =  ".extern " + sym->name() + " ";
+                  temp.writeNumber( (int64) sym->declaredAt() );
+               }
+
+               out->writeString( temp );
+               break;
+            }
+
             switch( sym->type() )
             {
-               case Symbol::tundef: out->writeString( ".extern" ); break;
                case Symbol::tglobal: out->writeString( ".global" ); break;
                case Symbol::tlocal: out->writeString( ".local" ); break;
                case Symbol::tparam: out->writeString( ".param" ); break;
                case Symbol::tfunc: out->writeString( ".func" ); break;
                case Symbol::tclass: out->writeString( ".class" ); break;
                case Symbol::tprop: out->writeString( ".prop" ); break;
-               case Symbol::tvar: out->writeString( ".propdef" ); break;
+               
+               case Symbol::tundef: out->writeString( ";---(INVALID) .undef" ); break;
+               case Symbol::tlocalundef: out->writeString( ";---(INVALID) .localundef" ); break;
+               case Symbol::textfunc: out->writeString( ";---(INVALID) .extfunc" ); break;
+               case Symbol::tconst: out->writeString( ";---(INVALID) .const" ); break;
+               case Symbol::timportalias: out->writeString( ";---(INVALID) .importalias" ); break;
+               case Symbol::tvar: out->writeString( ";---(INVALID) .var" ); break;
+               
                case Symbol::tinst:
                   out->writeString( ".instance $" );
                   out->writeString( sym->getInstance()->name() );
                   break;
-               case Symbol::tattribute: out->writeString( ".attrib" ); break;
             }
             out->writeString( " " + sym->name() );
             if( sym->declaredAt() != 0 ) {
@@ -1007,22 +1039,51 @@ void write_deptable( e_tabmode mode , Stream *out, Module *mod )
    out->writeString( ";--------------------------------------------\n" );
    out->writeString( "; Dependencies table\n" );
    out->writeString( ";--------------------------------------------\n" );
-   ListElement *iter = deps->begin();
-   while( iter != 0 )
+
+   MapIterator iter = deps->begin();
+
+   while( iter.hasCurrent() )
    {
-      str = (const String *) iter->data();
-      switch( mode )
+      const String *alias = *(const String **) iter.currentKey();
+      const ModuleDepData *data = *(const ModuleDepData **) iter.currentValue();
+      str = data->moduleName();
+
+      if ( data->isPrivate() )
       {
-         case e_mode_comment:
+         if ( mode == e_mode_comment )
+         {
             out->writeString( ";" );
-         break;
-         case e_mode_table:
-            out->writeString( ".load" );
-         break;
+
+            if ( data->isFile() )
+               out->writeString( " \"" + *str + "\"" );
+            else
+               out->writeString( " " + *str );
+
+            if ( *str != *alias )
+               out->writeString( " -> " + *alias );
+
+            out->writeString( " (private)" );
+         }
+      }
+      else {
+         switch( mode )
+         {
+            case e_mode_comment:
+               out->writeString( ";" );
+            break;
+            case e_mode_table:
+               out->writeString( ".load" );
+            break;
+         }
+
+         if ( data->isFile() )
+            out->writeString( " \"" + *str + "\"" );
+         else
+            out->writeString( " " + *str );
       }
 
-      out->writeString(  " " + *str + "\n" );
-      iter = iter->next();
+      out->writeString( "\n" );
+      iter.next();
    }
 
    out->writeString( "; ------  End of depend table -----------\n" );
@@ -1041,6 +1102,7 @@ void usage()
    stdOut->writeString( "\t-h: show this help\n" );
    stdOut->writeString( "\t-i: create an isomorphic version of the original assembly\n" );
    stdOut->writeString( "\tIf 'filename' is '-' or missing, read from stdin\n" );
+   stdOut->flush();
 }
 
 }
@@ -1050,8 +1112,7 @@ using namespace Falcon;
 int main( int argc, char *argv[] )
 {
    // initialize the engine
-   EngineData data;
-   Init( data );
+   Falcon::Engine::AutoInit autoInit;
 
    Stream *stdErr = stdErrorStream();
 
@@ -1071,7 +1132,7 @@ int main( int argc, char *argv[] )
          {
             switch ( option[ pos ] )
             {
-               case 'i': options.m_isomorphic = true; break;
+               case 'i': options.m_isomorphic = true; options.m_inline_str = true; break;
                case 's': options.m_dump_str = true; break;
                case 'l': options.m_lineinfo = true; break;
                case 'S': options.m_inline_str = true; break;
@@ -1083,6 +1144,7 @@ int main( int argc, char *argv[] )
                   stdErr->put( option[pos] );
                   stdErr->writeString( "'.\n" );
                   usage();
+                  stdErr->flush();
                   return 1;
             }
             ++pos;
@@ -1105,6 +1167,7 @@ int main( int argc, char *argv[] )
          stdErr->writeString( "faldisass: can't open file " );
          stdErr->writeString( options.m_fname );
          stdErr->writeString( "\n" );
+         stdErr->flush();
          return 1;
       }
       in = &inf;
@@ -1125,16 +1188,16 @@ int main( int argc, char *argv[] )
          stdErr->writeString( options.m_fname );
          stdErr->writeString( "\n" );
       }
+      stdErr->flush();
       return 1;
    }
 
+   global_symtab = &module->symbolTable();
+
    // for isomorphic we need the analysis.
    Stream *stdOut = DefaultTextTranscoder( new StdOutStream );
    t_labelMap labels;
-   t_funcMap functions;
    if ( options.m_isomorphic ) {
-      Analizer( module, labels );
-
 
       // print immediately the string table.
       write_strtable( e_mode_table, stdOut, module );
@@ -1143,33 +1206,68 @@ int main( int argc, char *argv[] )
       write_deptable( e_mode_table, stdOut, module );
 
       // and the symbol table.
-      write_symtable( e_mode_table, stdOut, &module->symbolTable() );
-
-      // also fetch the function map.
-      MapIterator iter = module->symbolTable().map().begin();
-      while( iter.hasCurrent() ) {
-         const Symbol *sym = *(const Symbol **) iter.currentValue();
-         if ( sym->isFunction() ) {
-            functions[ sym->getFuncDef()->offset() ] = sym;
-         }
-         else if ( sym->isClass() ) {
-            // we can generate the classess now, as the symbols have been
-            // already declared.
-            gen_class( stdOut, sym );
-         }
-
-         iter.next();
-      }
+      write_symtable( e_mode_table, stdOut, module );
    }
 
+   // now write MAIN, if it exists
+   Symbol *main = module->findGlobalSymbol( "__main__" );
+   if ( main != 0 )
+   {
+      current_symtab = global_symtab;
+      if ( options.m_isomorphic )
+      {
+         labels.clear();
+         Analizer( main->getFuncDef(), labels );
+      }
+      gen_code( module, main->getFuncDef(), stdOut, labels );
+   }
 
-   disassembler( module, stdOut, labels, functions );
+   // also fetch the function map.
+   MapIterator iter = module->symbolTable().map().begin();
+
+   while( iter.hasCurrent() )
+   {
+      const Symbol *sym = *(const Symbol **) iter.currentValue();
+      if ( sym->name() == "__main__" )
+      {
+         // already managed
+         iter.next();
+         continue;
+      }
+
+      if ( sym->isFunction() )
+      {
+         current_symtab = &sym->getFuncDef()->symtab();
+
+         if ( options.m_isomorphic )
+         {
+            labels.clear();
+            Analizer( sym->getFuncDef(), labels );
+            gen_function( module, sym, stdOut, labels );
+         }
+         else {
+            String tempOff;
+            tempOff.writeNumberHex( sym->getFuncDef()->basePC() );
+            stdOut->writeString( "\n; Generating function " + sym->name() + "(0x"+ tempOff + ")\n" );
+            gen_code( module, sym->getFuncDef(), stdOut, labels );
+         }
+
+
+      }
+      else if ( sym->isClass() && options.m_isomorphic ) {
+         // we can generate the classess now, as the symbols have been
+         // already declared.
+         gen_class( stdOut, sym );
+      }
+
+      iter.next();
+   }
 
    if ( options.m_dump_str ) {
       write_strtable( e_mode_comment, stdOut, module );
    }
    if ( options.m_dump_sym ) {
-      write_symtable( e_mode_comment, stdOut, &module->symbolTable() );
+      write_symtable( e_mode_comment, stdOut, module );
    }
    if ( options.m_dump_dep ) {
       write_deptable( e_mode_comment, stdOut, module );
