@@ -21,6 +21,7 @@
 #include <falcon/setup.h>
 #include <falcon/memory.h>
 #include <falcon/genericlist.h>
+#include <falcon/globals.h>
 
 #include <string.h>
 
@@ -286,7 +287,7 @@ bool ListTraits::owning() const
 
 namespace traits
 {
-	FALCON_DYN_SYM ListTraits t_List;
+	FALCON_DYN_SYM ListTraits &t_List() { static ListTraits lt; return lt; }
 }
 
 }
