@@ -178,36 +178,6 @@ void GenHAsm::gen_class( const StmtClass *cls )
       m_out->writeString( ".ctor $" + cd->constructor()->name() + "\n" );
    }
 
-   // write has and hasn't clauses
-   if ( ! cd->has().empty() )
-   {
-      m_out->writeString( ".has " );
-      ListElement *has_iter = cd->has().begin();
-      while( has_iter != 0 )
-      {
-         const Symbol *sym = (const Symbol *) has_iter->data();
-         m_out->writeString( "$" + sym->name() );
-         has_iter = has_iter->next();
-         if ( has_iter != 0 )
-            m_out->writeString( ", " );
-      }
-      m_out->writeString( "\n" );
-   }
-
-   if ( ! cd->hasnt().empty() ) {
-      m_out->writeString( ".hasnt " );
-      ListElement *hasnt_iter = cd->hasnt().begin();
-      while( hasnt_iter != 0 )
-      {
-         const Symbol *sym = (const Symbol *) hasnt_iter->data();
-         m_out->writeString( "$" + sym->name() );
-         hasnt_iter = hasnt_iter->next();
-         if ( hasnt_iter != 0 )
-            m_out->writeString( ", " );
-      }
-      m_out->writeString( "\n" );
-   }
-
    m_out->writeString( ".endclass\n" );
 }
 
@@ -377,12 +347,6 @@ void GenHAsm::gen_symbolTable( const Module *mod )
             m_out->writeString( temp );
          break;
 
-         case Symbol::tattribute:
-            temp =  ".attrib " + sym->name() + " ";
-            temp.writeNumber( (int64) sym->declaredAt() );
-            m_out->writeString( temp );
-         break;
-
          case Symbol::timportalias:
             {
                ImportAlias* ia = sym->getImportAlias();
@@ -542,17 +506,10 @@ void GenHAsm::gen_statement( const Statement *stmt )
       case Statement::t_continue:
       {
          fassert( ! m_loops.empty() );
-         int target_pos = (int) m_loops.end()->iData();
-         // was this a for/in loop? -- in that case, the number is negative.
-         bool bIsForin;
-         if( target_pos < 0 )
-         {
-            bIsForin = true;
-            target_pos = -target_pos;
-         }
-         else {
-            bIsForin = false;
-         }
+         LoopInfo* loop = (LoopInfo*)  m_loops.back();
+         int target_pos = loop->m_id;
+
+
          // there is a TRY statement above this one. If the TRY level is "inner"
          // ( try is inside the loop we are going to break ) we have to also pop
          // the TRY value while restarting the loop. If the TRY is outside the loop,
@@ -575,16 +532,46 @@ void GenHAsm::gen_statement( const Statement *stmt )
          if ( stmt->type() == Statement::t_continue )
          {
             const StmtContinue *cont = static_cast<const StmtContinue *>( stmt );
-            if( bIsForin )
+            // is a continue dropping? -- we must generate a TRDN opcode.
+            if( cont->dropping() )
             {
-               m_out->writeString( "\tTRAN\t_loop_begin_" + loopStr + ", _loop_tral_" + loopStr );
-               if( cont->dropping() )
-                  m_out->writeString( ", 1\n" ); // 1 -- delete
+               fassert( loop->m_loop->type() == Statement::t_forin );
+
+               // when generating the last element, TRDN is simpler.
+               if( loop->m_isForLast )
+               {
+                  m_out->writeString( "\tTRDN\t_loop_end_" + loopStr + ", 0, 0\n" );
+               }
                else
-                  m_out->writeString( ", 0\n" ); // 0 just TRAN
+               {
+                  StmtForin* fin = (StmtForin* ) loop->m_loop;
+                  String svars;
+                  svars.writeNumber((int64) fin->dest()->size() );
+                  m_out->writeString( "\tTRDN\t_loop_begin_" + loopStr + ", " +
+                                      "_loop_end_" + loopStr +
+                                      ", "+ svars +"\n" );
+                  ListElement *it_t = fin->dest()->begin();
+
+                  // first generates an array of references
+                  while( it_t != 0 ) {
+                     // again, is the compiler that must make sure of this...
+                     const Value *val = (const Value *) it_t->data();
+                     fassert( val->isSimple() );
+
+                     m_out->writeString( "\tNOP \t" );
+                     gen_operand( val );
+                     m_out->writeString( "\n" );
+
+                     it_t = it_t->next();
+                  }
+               }
             }
             else {
-               m_out->writeString( "\tJMP \t_loop_next_" + loopStr + "\n" );
+               // when generating the last element, continue == break.
+               if( loop->m_isForLast )
+                  m_out->writeString( "\tJMP \t_loop_end_" + loopStr + "\n" );
+               else
+                  m_out->writeString( "\tJMP \t_loop_next_" + loopStr + "\n" );
             }
          }
          else
@@ -603,29 +590,6 @@ void GenHAsm::gen_statement( const Statement *stmt )
       }
       break;
 
-      case Statement::t_pass:
-      {
-         const StmtPass *pass = static_cast< const StmtPass *>( stmt );
-         String mode = pass->saveIn() == 0 ? "\tPASS\t" : "\tPSIN\t" ;
-         if ( pass->called()->isSimple() )
-         {
-            m_out->writeString( mode );
-            gen_operand( pass->called() );
-         }
-         else {
-            gen_value( pass->called() );
-            m_out->writeString( mode + "A" );
-         }
-         m_out->writeString( "\n" );
-
-         if ( pass->saveIn() != 0 )
-         {
-            gen_load_from_A( pass->saveIn() );
-         }
-      }
-      break;
-
-
       case Statement::t_autoexp:
       {
          const StmtExpression *val = static_cast<const StmtExpression *>( stmt );
@@ -720,101 +684,6 @@ void GenHAsm::gen_statement( const Statement *stmt )
       }
       break;
 
-      case Statement::t_give:
-      {
-         const StmtGive *give = static_cast<const StmtGive *>( stmt );
-         const ArrayDecl *give_to = give->objects();
-         const ListElement *elem = give_to->begin();
-
-         while( elem != 0 )
-         {
-            const Value *object = (const Value *) elem->data();
-
-            if ( ! object->isSimple() )
-            {
-               gen_complex_value( object );
-            }
-
-            const ArrayDecl *attribs = give->attributes();
-            ListElement *iter = attribs->begin();
-            bool pushed = false;
-            bool peek = false;
-
-            while( iter != 0 )
-            {
-               const Value *val = (const Value *) iter->data();
-               String mode;
-               ListElement *iter_next = iter->next();
-
-               if ( val->isExpr() && val->asExpr()->type() == Expression::t_not ) {
-                  val = val->asExpr()->first();
-                  mode = "\tGIVN\t";
-               }
-               else
-                  mode = "\tGIVE\t";
-
-
-               if( val->isSimple() )
-               {
-                  if ( object->isSimple() ) {
-                     m_out->writeString( mode );
-                     gen_operand( object );
-                     m_out->writeString( ", " );
-                     gen_operand( val );
-                  }
-                  else {
-                     if ( peek ) {
-                        if ( iter_next != 0 )
-                           m_out->writeString( "\tPEEK\tA\n" );
-                        else {
-                           m_out->writeString( "\tPOP\tA\n" );
-                           pushed = false;
-                        }
-                        peek = false;
-                     }
-                     m_out->writeString( mode + "A, " );
-                     gen_operand( val );
-                  }
-               }
-               else {
-                  if ( object->isSimple() ) {
-                     gen_value( val );
-                     m_out->writeString( mode );
-                     gen_operand( object );
-                     m_out->writeString( ", A" );
-                  }
-                  else {
-                     if ( ! pushed ) {
-                        m_out->writeString( "\tPUSH A\n" );
-                        pushed = true;
-                     }
-
-                     gen_value( val );
-                     if ( iter_next != 0 )
-                        m_out->writeString( "\tPEEK\tB\n" );
-                     else {
-                        m_out->writeString( "\tPOP\tB\n" );
-                        pushed = false;
-                     }
-                     peek = true;
-
-                     m_out->writeString( mode + "B, A" );
-                  }
-               }
-
-               m_out->writeString( "\n" );
-               iter = iter_next;
-            }
-            if ( pushed ) {
-               m_out->writeString( "\tIPOP\t1\n" );
-            }
-
-            elem = elem->next();
-         }
-      }
-      break;
-
-
       case Statement::t_unref:
       {
          const StmtUnref *ass = static_cast<const StmtUnref *>(stmt);
@@ -986,7 +855,8 @@ void GenHAsm::gen_statement( const Statement *stmt )
          const StmtWhile *elem = static_cast<const StmtWhile *>( stmt );
 
          int branch = m_loop_id++;
-         m_loops.pushBack( (void *) branch );
+         LoopInfo loop( branch, elem );
+         m_loops.pushBack( (void *) &loop );
          String branchStr;
          branchStr.writeNumber( (int64) branch );
          m_out->writeString( "_loop_next_" + branchStr + ":\n" );
@@ -1013,6 +883,49 @@ void GenHAsm::gen_statement( const Statement *stmt )
       }
       break;
 
+      case Statement::t_loop:
+      {
+         const StmtLoop *elem = static_cast<const StmtLoop* >( stmt );
+
+         int branch = m_loop_id++;
+         LoopInfo loop( branch, elem );
+         m_loops.pushBack( (void *) &loop );
+         String branchStr;
+         branchStr.writeNumber( (int64) branch );
+         m_out->writeString( "_loop_begin_" + branchStr + ":\n" );
+
+         if ( elem->condition() == 0 )
+            m_out->writeString( "_loop_next_" + branchStr + ":\n" );
+
+         gen_block( &elem->children() );
+
+         if ( elem->condition() == 0 )
+         {
+            // endless loop
+            m_out->writeString( "\tJMP \t_loop_begin_" + branchStr + "\n" );
+         }
+         else {
+            m_out->writeString( "_loop_next_" + branchStr + ":\n" );
+
+            if ( ! elem->condition()->isTrue() )
+            {
+               if ( elem->condition()->isSimple() ) {
+                  m_out->writeString( "\tIFF \t_loop_begin_" + branchStr + ", " );
+                  gen_operand( elem->condition() );
+               }
+               else {
+                  gen_complex_value( elem->condition() );
+                  m_out->writeString( "\tIFF \t_loop_begin_" + branchStr + ", A" );
+               }
+               m_out->writeString( "\n" );
+            }
+         }
+         // if it's true, terminate immediately
+
+         m_out->writeString( "_loop_end_" + branchStr + ":\n" );
+         m_loops.popBack();
+      }
+      break;
 
       case Statement::t_propdef:
       {
@@ -1032,103 +945,43 @@ void GenHAsm::gen_statement( const Statement *stmt )
       case Statement::t_forin:
       {
          const StmtForin *loop = static_cast<const StmtForin *>( stmt );
-         int neededVars = 0;  // vars pushed by reference for target expansion
 
          int loopId = m_loop_id++;
 
-         // we push the negative number to signal this is a for/in loop
-         m_loops.pushBack( (void *) (-loopId) );
+         LoopInfo li( loopId, loop );
+         m_loops.pushBack( (void *) &li );
 
          String loopStr;
          loopStr.writeNumber( (int64) loopId );
 
-         // ================================== Trav block
-         // PSHR  $var1
-         // ...
-         // PSHR  $varN
-         // TRAV Jump On Failure, NumberOfVars (immediate int), SOURCE (or A),
-         //            ----or----
-         // TRAV Jump On Failure, $target, SOURCE (or A)
-         //    ---> Push immediate int (counter) or object (iterator)
-         //    ---> Push PSHR $target / immediate integer (num of vars)
-         //    ---> Push source
+         String snv;
+         snv.writeNumber( (int64) loop->dest()->size() );
 
-         //    [FORFIRST Block]
-         //   ...
-         // begin:
-         //    [loop body]
-         //    [If we have a forlast block  --
-         //      TRAL (last)]
-         // [FORMIDDLE block]
-         //
-         // TRAN begin
-         // last:
-         //   [last block]
-         // exit:
-         // ipop 3 + pushed vars.
-
-         if ( loop->dest()->isSimple() ) {
-            if ( loop->source()->isSimple() ) {
-               m_out->writeString( "\tTRAV\t" );
-               m_out->writeString( "_p_loop_end_" + loopStr + ", " );
-               gen_operand( loop->dest() );
-               m_out->writeString( ", " );
-               gen_operand( loop->source() );
-               m_out->writeString( "\n" );
-            }
-            else {
-               gen_value( loop->source() );
-               m_out->writeString( "\tTRAV\t_p_loop_end_" + loopStr + ", " );
-               gen_operand( loop->dest() );
-               m_out->writeString( ", A\n" );
-            }
+         if ( loop->source()->isSimple() ) {
+            m_out->writeString( "\tTRAV\t_p_loop_end_" + loopStr + ", " );
+            m_out->writeString( snv + ", " );
+            gen_operand( loop->source() );
+            m_out->writeString( "\n" );
          }
          else {
-            if( loop->dest()->type() == Value::t_array_decl ) {
-               ListElement *it_t = loop->dest()->asArray()->begin();
+            gen_value( loop->source() );
+            m_out->writeString( "\tTRAV\t_p_loop_end_" + loopStr + ", " );
+            m_out->writeString( snv + ", A\n" );
+         }
 
-               // first generates an array of references
-               while( it_t != 0 ) {
-                  // again, is the compiler that must make sure of this...
-                  const Value *val = (const Value *) it_t->data();
-                  fassert( val->isSimple() );
+         ListElement* it_t = loop->dest()->begin();
 
-                  m_out->writeString( "\tPSHR\t" );
-                  gen_operand( val );
-                  m_out->writeString( "\n" );
+         // first generates an array of references
+         while( it_t != 0 ) {
+            // again, is the compiler that must make sure of this...
+            const Value *val = (const Value *) it_t->data();
+            fassert( val->isSimple() );
 
-                  ++neededVars;
-                  it_t = it_t->next();
-               }
+            m_out->writeString( "\tNOP \t" );
+            gen_operand( val );
+            m_out->writeString( "\n" );
 
-               String snv;
-               snv.writeNumber( (int64) neededVars );
-               if ( loop->source()->isSimple() ) {
-                  m_out->writeString( "\tTRAV\t_p_loop_end_" + loopStr +", " );
-                  m_out->writeString( snv + ", " );
-                  gen_operand( loop->source() );
-                  m_out->writeString( "\n" );
-               }
-               else {
-                  gen_value( loop->source() );
-                  m_out->writeString( "\tTRAV\t_p_loop_end_" + loopStr +", " );
-                  m_out->writeString( snv + ", A\n" );
-               }
-            }
-            else {
-               gen_value( loop->dest() );
-               if ( loop->source()->isSimple() ) {
-                  m_out->writeString( "\tTRAV\t_p_loop_end_" + loopStr +", A, " );
-                  gen_operand( loop->source() );
-                  m_out->writeString( "\n" );
-               }
-               else {
-                  m_out->writeString( "\tPUSH\tA\n" ); // so we save the expression in B
-                  gen_value( loop->source() );
-                  m_out->writeString( "\tPOP \tB\n" ); // so we save the expression in B
-                  m_out->writeString( "\tTRAV\t_p_loop_end_" + loopStr + ", B, A\n" );
-               }
-            }
+            it_t = it_t->next();
          }
 
          // have we got a "first" block?
@@ -1146,25 +999,37 @@ void GenHAsm::gen_statement( const Statement *stmt )
          if( ! loop->middleBlock().empty() ) {
             // skip it for the last element
             m_out->writeString( "\tTRAL\t_loop_tral_" + loopStr + "\n" );
-
             gen_block( &loop->middleBlock() );
          }
 
          m_out->writeString( "_loop_next_" + loopStr + ":\n" );
-         m_out->writeString( "\tTRAN\t_loop_begin_" + loopStr + ", _loop_tral_" + loopStr + ", 0\n" );
+         m_out->writeString( "\tTRAN\t_loop_begin_" + loopStr + ", "+ snv +",\n");
+         it_t = loop->dest()->begin();
+
+         // first generates an array of references
+         while( it_t != 0 ) {
+           // again, is the compiler that must make sure of this...
+           const Value *val = (const Value *) it_t->data();
+           fassert( val->isSimple() );
+
+           m_out->writeString( "\tNOP \t" );
+           gen_operand( val );
+           m_out->writeString( "\n" );
+
+           it_t = it_t->next();
+         }
 
          // generate the last block
          m_out->writeString( "_loop_tral_" + loopStr + ":\n" );
          if( ! loop->lastBlock().empty() ) {
             // and the last time...
+            li.m_isForLast = true;
             gen_block( &loop->lastBlock() );
          }
 
          // create break landing code:
          m_out->writeString( "_loop_end_" + loopStr + ":\n" );
-         String varToPop;
-         varToPop.writeNumber( (int64) (3 + neededVars) );
-         m_out->writeString( "\tIPOP\t" + varToPop + "\n" );
+         m_out->writeString( "\tIPOP\t1\n" );
          // internal loop out used by TRAV, TRAN and TRAL
          m_out->writeString( "_p_loop_end_" + loopStr + ":\n" );
 
@@ -1182,7 +1047,7 @@ void GenHAsm::gen_statement( const Statement *stmt )
          if ( m_loops.empty() )
             m_trys.pushBack( (void *) -1 );
          else
-            m_trys.pushBack( m_loops.back() );
+            m_trys.pushBack( ((LoopInfo*) m_loops.back())->m_id );
 
          int branch = m_try_id++;
          String branchStr;
@@ -1616,10 +1481,6 @@ void GenHAsm::gen_operand( const Value *stmt )
          m_out->writeString( "S1" );
       break;
 
-      case Value::t_sender:
-         m_out->writeString( "S2" );
-      break;
-
       default:
          m_out->writeString( "???" );
    }
@@ -1894,31 +1755,61 @@ void GenHAsm::gen_expression( const Expression *exp, t_valType &xValue )
       case Expression::t_lambda:
       {
          xValue = l_value;
-         m_out->writeString( "\tSTO \tA, $" + exp->first()->asSymbol()->name() + "\n" );
+         if( exp->second() != 0 )
+         {
+            // we must create the lambda closure
+            int size = 0;
+            ListElement *iter = exp->second()->asArray()->begin();
+            while( iter != 0 )
+            {
+               const Value *val = (Value *) iter->data();
+               if( val->isSimple() ) {
+                  m_out->writeString( "\tPSHR\t" );
+                  gen_operand( val );
+               }
+               else {
+                  gen_complex_value( val );
+                  m_out->writeString( "\tPSHR\tA" );
+               }
+               m_out->writeString( "\n" );
+               
+               size++;
+               iter = iter->next();
+            }
+
+            String temp; temp.writeNumber((int64) size);
+            m_out->writeString( "\tCLOS\t" + temp +
+                  ", A, $" + exp->first()->asSymbol()->name() + "\n" );
+         }
+         else
+         {
+            m_out->writeString( "\tSTO \tA, $" + exp->first()->asSymbol()->name() + "\n" );
+
+         }
       }
       return;
-      
+
       case Expression::t_oob:
          xValue = l_value;
          m_out->writeString( "\tOOB  \t1, " );
             gen_operand( exp->first() );
             m_out->writeString( "\n" );
       return;
-      
+
       case Expression::t_deoob:
          xValue = l_value;
          m_out->writeString( "\tOOB  \t0, " );
             gen_operand( exp->first() );
             m_out->writeString( "\n" );
       return;
-      
+
       case Expression::t_xoroob:
          xValue = l_value;
          m_out->writeString( "\tOOB  \t2, " );
             gen_operand( exp->first() );
             m_out->writeString( "\n" );
       return;
-      
+
       case Expression::t_isoob:
          xValue = l_value;
          m_out->writeString( "\tOOB  \t3, " );
@@ -2144,11 +2035,14 @@ void GenHAsm::gen_load( const Value *target, const Value *source )
             gen_store_to_deep( "STP", exp->first(), exp->second(), source );
          }
       }
-      else if ( target->type() == Value::t_array_decl ) {
+      else if ( target->type() == Value::t_array_decl )
+      {
+         const ArrayDecl *tarr = target->asArray();
+
          // if the source is also an array, fine, we have a 1:1 assignment.
          if ( source->type() == Value::t_array_decl ) {
             const ArrayDecl *sarr = source->asArray();
-            const ArrayDecl *tarr = target->asArray();
+
             ListElement *it_s = sarr->begin();
             ListElement *it_t = tarr->begin();
 
@@ -2164,24 +2058,36 @@ void GenHAsm::gen_load( const Value *target, const Value *source )
             fassert( it_s == 0 && it_t == 0 );
          }
          // we must generate an unpack request
-         else {
-
+         else
+         {
+            String ssize;
+            ssize.writeNumber( (int64) tarr->size() );
             // then unpack the source in the array.
-            if ( source->isSimple() ) {
-	            int size = gen_refArray( target->asArray(), false );
-					String instr = "\tUNPS\t";
-					instr.writeNumber( (int64) size );
-               m_out->writeString( instr );
-					m_out->writeString( ", " );
+            if ( source->isSimple() )
+            {
+               m_out->writeString( "\tLDAS\t" + ssize + ", " );
                gen_operand( source );
                m_out->writeString( "\n" );
             }
             else {
-               gen_complex_value( source );
-               int size = gen_refArray( target->asArray(), false );
-					String instr = "\tUNPS\t";
-					instr.writeNumber( (int64) size );
-	            m_out->writeString( instr + ", A\n" );
+					gen_complex_value( source );
+					m_out->writeString( "\tLDAS\t" + ssize + ", A\n" );
+            }
+
+            ListElement *it_t = tarr->end();
+            while( it_t != 0 ) {
+               const Value *t = (const Value *) it_t->data();
+               if( t->isSimple() )
+               {
+                  m_out->writeString( "\tPOP \t" );
+                  gen_operand( t );
+                  m_out->writeString( "\n" );
+               }
+               else {
+                  m_out->writeString( "\tPOP \tB\n" );
+                  gen_load_from_reg( t, "B" );
+               }
+               it_t = it_t->prev();
             }
          }
       }
@@ -2379,35 +2285,42 @@ void GenHAsm::gen_load_from_reg( const Value *target, const char *reg )
          else if ( exp->type() == Expression::t_obj_access ) {
             gen_store_to_deep_reg( "STP", exp->first(), exp->second(), reg );
          }
+         else {
+            m_out->writeString( "\tPUSH\t" + regStr + "\n" );
+            gen_expression( exp );
+            m_out->writeString( "\tPOP \tB\n" );
+            m_out->writeString( "\tLD  \tA,B\n" );
+         }
       }
-      else if ( target->type() == Value::t_array_decl ) {
+      else if ( target->type() == Value::t_array_decl )
+      {
          // if the source is also an array, fine, we have a 1:1 assignment.
          const ArrayDecl *tarr = target->asArray();
-         ListElement *it_t = tarr->begin();
-         int size = 0;
+         String ssize;
+         ssize.writeNumber((int64) tarr->size() );
 
-         m_out->writeString( "\tPUSH\t" + regStr + "\n" );
+         // push the source array on the stack.
+         m_out->writeString( "\tLDAS\t" + ssize + ", " + regStr + "\n" );
 
-         // first generates an array of references
-         while( it_t != 0 ) {
-            // again, is the compiler that must make sure of this...
+         // and load each element back in the array
+         ListElement *it_t = tarr->end();
+
+         // Now load each element by popping it.
+         while( it_t != 0 )
+         {
             const Value *val = (const Value *) it_t->data();
-            fassert( val->isSimple() );
-
-            m_out->writeString( "\tPSHR\t" );
-            gen_operand( val );
-            m_out->writeString( "\n" );
-
-            ++size;
-            it_t = it_t->next();
+            if( val->isSimple() )
+            {
+               m_out->writeString( "\tPOP \t" );
+               gen_operand( val );
+               m_out->writeString( "\n" );
+            }
+            else {
+               m_out->writeString( "\tPOP \tB\n" );
+               gen_load_from_reg( val, "B" );
+            }
+            it_t = it_t->prev();
          }
-         String sizeStr;
-         sizeStr.writeNumber( (int64) size );
-
-         m_out->writeString( "\tGENA\t" + sizeStr + "\n" );
-
-         m_out->writeString( "\tPOP \tB" );
-         m_out->writeString( "\tUNPK\tA, B" );
       }
    }
 }
