@@ -21,12 +21,13 @@
 #include <falcon/fstream.h>
 #include <falcon/transcoding.h>
 #include <falcon/stdstreams.h>
+#include <falcon/streambuffer.h>
 
 namespace Falcon {
 
 Stream *stdInputStream()
 {
-   StdInStream *stdin = new StdInStream;
+   Stream *stdin = new StreamBuffer( new StdInStream );
    String enc;
 
    Falcon::Transcoder *coder;
@@ -47,9 +48,11 @@ Stream *stdInputStream()
    return coder;
 }
 
+
+
 Stream *stdOutputStream()
 {
-   StdOutStream *stdout = new StdOutStream;
+   Stream *stdout = new StreamBuffer( new StdOutStream );
    String enc;
 
    Falcon::Transcoder *coder;
@@ -73,7 +76,7 @@ Stream *stdOutputStream()
 
 Stream *stdErrorStream()
 {
-   StdErrStream *stderr = new StdErrStream;
+   Stream *stderr = new StreamBuffer( new StdErrStream );
    String enc;
 
    Falcon::Transcoder *coder;
