@@ -142,7 +142,7 @@ bool Value::isEqualByValue( const Value &other ) const
          case Value::t_range_decl:
             if( other.asRange()->rangeStart() == asRange()->rangeStart() &&
                   ( (asRange()->isOpen() && other.asRange()->isOpen()) ||
-                    (! asRange()->isOpen() && ! other.asRange()->isOpen() 
+                    (! asRange()->isOpen() && ! other.asRange()->isOpen()
                        && other.asRange()->rangeEnd() == asRange()->rangeEnd())
                   )
                )
@@ -261,6 +261,10 @@ VarDef *Value::genVarDef()
          def = new Falcon::VarDef( asString() );
       break;
 
+      case Falcon::Value::t_imm_bool:
+         def = new Falcon::VarDef( asBool() );
+      break;
+
       default:
          def = 0;  // set a nil expression
    }
@@ -394,6 +398,30 @@ Expression::~Expression()
    delete m_third;
 }
 
+bool Expression::isStandAlone() const
+{
+   switch( m_operator )
+   {
+      case t_eval: case t_funcall: case t_assign:
+      case t_aadd: case t_asub: case t_amul: case t_adiv:
+      case t_amod: case t_apow:
+      case t_aband: case t_abor: case t_abxor:
+      case t_ashl: case t_ashr:
+      case t_pre_inc: case t_post_inc: case t_pre_dec: case t_post_dec:
+         return true;
+
+      // and are or are meaningful if their second element is an effective expression.
+      // the first one needs not (i.e. may be a variable), and if the first one is
+      // but the second one is not, then the second part does nothing, and we're
+      // right in raising an error as there's something wrong.
+      case t_and: case t_or:
+         return m_second->isExpr() && m_second->asExpr()->isStandAlone();
+
+      default:
+         return false;
+   }
+}
+
 //================================================
 // Statement list
 //
@@ -567,28 +595,6 @@ Statement *StmtLaunch::clone() const
    return new StmtLaunch( *this );
 }
 
-// StmtPass statement
-//
-
-StmtPass::StmtPass( const StmtPass &other ):
-   Statement( other )
-{
-   m_called = other.m_called == 0 ? 0 : new Value( *other.m_called );
-   m_saveRetIn = other.m_saveRetIn == 0 ? 0 : new Value( *other.m_saveRetIn );
-}
-
-StmtPass::~StmtPass()
-{
-   delete m_called;
-   delete m_saveRetIn;
-}
-
-Statement *StmtPass::clone() const
-{
-   return new StmtPass( *this );
-}
-
-
 
 // StmtGive statement
 //
@@ -653,9 +659,15 @@ StmtConditional::~StmtConditional()
    delete m_condition;
 }
 
+// StmtLoop statement
+//
+Statement *StmtLoop::clone() const
+{
+   return new StmtLoop( *this );
+}
+
 // StmtWhile statement
 //
-
 Statement *StmtWhile::clone() const
 {
    return new StmtWhile( *this );
@@ -694,7 +706,7 @@ StmtForin::StmtForin( const StmtForin &other ):
    m_middle( other.m_middle )
 {
    m_source = other.m_source == 0 ? 0 : new Value( * other.m_source );
-   m_dest = other.m_dest == 0 ? 0 : new Value( * other.m_dest );
+   m_dest = other.m_dest == 0 ? 0 : new ArrayDecl( * other.m_dest );
 }
 
 StmtForin::~StmtForin()
