@@ -35,6 +35,9 @@
 namespace Falcon {
 namespace Sys {
 
+void _dummy_ctrl_c_handler()
+{
+}
 
 numeric _seconds()
 {
@@ -76,7 +79,6 @@ void _tempName( String &res )
 {
    static bool first = true;
    const char *temp_dir;
-   char *fname;
    struct stat st;
 
    if( first ) {
