@@ -36,6 +36,24 @@
 namespace Falcon {
 namespace Sys {
 
+static BOOL CtrlHandler(DWORD dwCtrlType)
+{
+    if( CTRL_C_EVENT == dwCtrlType)
+    {
+       // just terminate the process.
+       exit(0);
+    }
+    return FALSE;
+}
+
+void _dummy_ctrl_c_handler()
+{
+    SetConsoleCtrlHandler(
+      (PHANDLER_ROUTINE) CtrlHandler,
+      TRUE); 
+}
+
+
 numeric SYSTEMTIME_TO_SECONDS( const SYSTEMTIME &st )
 {
    int secsAt[12];
@@ -129,7 +147,7 @@ void _tempName( String &res )
 	{
 		char *bufname_c = (char *) wct;
 		if( temp_dir.toCString( bufname_c, tempLen ) > 0 )
-			attribs = GetFileAttributes( bufname_c );
+			attribs = GetFileAttributesA( bufname_c );
 	}
 
    if ( GetLastError() != 0 || ((attribs & FILE_ATTRIBUTE_DIRECTORY) == 0) )
@@ -280,18 +298,33 @@ void falconConvertWinFname( String &ret )
 {
    int size = ret.length();
 	ret.bufferize();
+   bool prefix = false;
+
    for( int i = 0; i < size; i ++ ) {
       int chr = ret.getCharAt( i );
       if( chr == '\\' )
          ret.setCharAt( i, '/' );
+      else if ( chr == ':' )
+         prefix = true;
    }
+
+   if( prefix && ret.getCharAt(0) != '/' )
+      ret.prepend('/');
 }
 
 void falconToWin_fname( const String &path, const String &extra, String &result )
 {
-   result = path + extra;
-	result.bufferize();
+   // remove the leading "/" in url unit format (/unit:/path...)
+   if ( path.getCharAt(0) == '/' && path.find(":") != 0 )
+   {
+      result = path.subString(1) + extra;
+   }
+   else {
+      result = path + extra;
+	   result.bufferize();
+	}
 
+   // Changes slashes into backslashes
    for( unsigned int i = 0; i < result.length(); i ++ ) {
       int chr = result.getCharAt( i );
       if( chr == '/' )
