@@ -24,6 +24,7 @@
 #include <falcon/item.h>
 
 #include <stdio.h>
+#include <string.h>
 #include <time.h>
 
 static const char *RFC_2822_days[] = { "Mon","Tue", "Wed","Thu","Fri","Sat","Sun" };
@@ -211,7 +212,7 @@ const char *TimeStamp::getRFC2822_MonthName( int16 month )
 
 int16 TimeStamp::getRFC2822_WeekDay( const char *name )
 {
-   for ( int16 i = 0; i < sizeof( RFC_2822_days ) / sizeof( char *); i ++ )
+   for ( uint32 i = 0; i < sizeof( RFC_2822_days ) / sizeof( char *); i ++ )
    {
       if ( strncmp( RFC_2822_days[i], name, 3) == 0 )
          return i;
@@ -221,7 +222,7 @@ int16 TimeStamp::getRFC2822_WeekDay( const char *name )
 
 int16 TimeStamp::getRFC2822_Month( const char *name )
 {
-   for ( int16 i = 0; i < sizeof( RFC_2822_months ) / sizeof( char *); i ++ )
+   for ( uint32 i = 0; i < sizeof( RFC_2822_months ) / sizeof( char *); i ++ )
    {
       if ( strncmp( RFC_2822_months[i], name, 3) == 0 )
          return i+1;
@@ -530,14 +531,6 @@ void TimeStamp::fromLongFormat( int64 lf )
    m_year = (int16) lf;
 }
 
-numeric TimeStamp::asSeconds() const
-{
-   return 0;
-}
-
-void TimeStamp::fromSeconds( numeric seconds )
-{
-}
 
 void TimeStamp::add( const TimeStamp &ts )
 {
@@ -549,7 +542,7 @@ void TimeStamp::add( const TimeStamp &ts )
 
    if ( m_timezone != ts.m_timezone && m_timezone != tz_NONE && ts.m_timezone != tz_NONE )
    {
-      int16 hours, mins, ts_hours, ts_mins;
+      int16 hours=0, mins=0, ts_hours=0, ts_mins=0;
       ts.getTZDisplacement( ts_hours, ts_mins );
       getTZDisplacement( hours, mins );
       m_hour += hours - ts_hours;
@@ -625,7 +618,7 @@ void TimeStamp::distance( const TimeStamp &ts )
    m_year = 0;
    m_month = 0;
 
-   int tday = days;
+   //int tday = days;
    int thour = endDate->m_hour;
    int tminute = endDate->m_minute;
    int tsecond = endDate->m_second;
@@ -633,7 +626,7 @@ void TimeStamp::distance( const TimeStamp &ts )
 
    if ( m_timezone != ts.m_timezone && m_timezone != tz_NONE && ts.m_timezone != tz_NONE )
    {
-      int16 hours, mins, ts_hours, ts_mins;
+      int16 hours=0, mins=0, ts_hours=0, ts_mins=0;
       ts.getTZDisplacement( ts_hours, ts_mins );
       getTZDisplacement( hours, mins );
       // if ts bigger (positive distance) we must add the difference between TS timezone and us
@@ -784,8 +777,8 @@ int32 TimeStamp::compare( const TimeStamp &ts ) const
       if ( m_day > ts.m_day  ) return 1;
    }
    else {
-      int16 hdisp, mdisp;
-      int16 ts_hdisp, ts_mdisp;
+      int16 hdisp=0, mdisp=0;
+      int16 ts_hdisp=0, ts_mdisp=0;\
 
       getTZDisplacement( hdisp, mdisp );
       ts.getTZDisplacement( ts_hdisp, ts_mdisp );
@@ -872,24 +865,18 @@ void TimeStamp::toString( String &target ) const
 
 bool TimeStamp::toString( String &target, const String &fmt ) const
 {
-   char fmtBuf[256];
-
-   if( ! fmt.toCString( fmtBuf, 256 )  )
-   {
-      return false;
-   }
-
+   AutoCString cfmt( fmt );
    struct tm theTime;
 
    theTime.tm_sec = m_second;
    theTime.tm_min = m_minute;
    theTime.tm_hour = m_hour;
    theTime.tm_mday = m_day;
-   theTime.tm_mon = m_month;
+   theTime.tm_mon = m_month-1;
    theTime.tm_year = m_year - 1900;
 
    char timeTgt[512];
-   if( strftime( timeTgt, 512, fmtBuf, &theTime) != 0 )
+   if( strftime( timeTgt, 512, cfmt.c_str(), &theTime) != 0 )
    {
       target.bufferize( timeTgt );
 
@@ -968,7 +955,7 @@ void TimeStamp::changeTimezone( TimeZone tz )
       return;
 
    // get the relative total shift.
-   int16 currentHour, currentMin, newHour, newMin;
+   int16 currentHour=0, currentMin=0, newHour=0, newMin=0;
    getTZDisplacement( tz, newHour, newMin );
    getTZDisplacement( m_timezone, currentHour, currentMin );
 
@@ -980,70 +967,7 @@ void TimeStamp::changeTimezone( TimeZone tz )
    m_timezone = tz;
 }
 
-bool TimeStamp::isReflective() const
-{
-   return true;
-}
-
-void TimeStamp::getProperty( VMachine *, const String &propName, Item &prop )
-{
-   if( propName == "year" )
-   {
-      prop = (int64) m_year;
-   }
-   else if( propName == "month" ) {
-      prop = (int64) m_month;
-   }
-   else if( propName == "day" ) {
-      prop = (int64) m_day;
-   }
-   else if( propName == "hour" ) {
-      prop = (int64) m_hour;
-   }
-   else if( propName == "minute" ) {
-      prop = (int64) m_minute;
-   }
-   else if( propName == "second" ) {
-      prop = (int64) m_second;
-   }
-   else if( propName == "msec" ) {
-      prop = (int64) m_msec;
-   }
-   else if( propName == "timezone" ) {
-      prop = (int64) m_timezone;
-   }
-}
-
-void TimeStamp::setProperty( VMachine *, const String &propName, Item &prop )
-{
-   if( propName == "year" )
-   {
-      m_year = (int16) prop.forceInteger();
-   }
-   else if( propName == "month" ) {
-      m_month = (int16) prop.forceInteger();
-   }
-   else if( propName == "day" ) {
-      m_day = (int16) prop.forceInteger();
-   }
-   else if( propName == "hour" ) {
-      m_hour = (int16) prop.forceInteger();
-   }
-   else if( propName == "minute" ) {
-      m_minute = (int16) prop.forceInteger();
-   }
-   else if( propName == "second" ) {
-      m_second = (int16) prop.forceInteger();
-   }
-   else if( propName == "msec" ) {
-      m_msec = (int16) prop.forceInteger();
-   }
-   else if( propName == "timezone" ) {
-      m_timezone = (TimeZone)( prop.forceInteger()%32);
-   }
-}
-
-UserData *TimeStamp::clone() const
+FalconData *TimeStamp::clone() const
 {
    return new TimeStamp( *this );
 }
