@@ -54,12 +54,12 @@ public:
 };
 
 /** File stream base class.
-   The FileStream class is the base for the system-specific stream handlers.
+   The BaseFileStream class is the base for the system-specific stream handlers.
    It provides an interface to the system functions that work with
    system streams, as files or standard streams.
 */
 
-class FALCON_DYN_CLASS GenericStream: public Stream
+class FALCON_DYN_CLASS BaseFileStream: public Stream
 {
 public:
    /** Open mode. */
@@ -90,7 +90,7 @@ public:
 
       e_aAll = 0777,
       e_aReadOnly = 0444,
-      e_aHidden = 0600,
+      e_aHidden = 0600
    } t_attributes;
 
 protected:
@@ -100,18 +100,14 @@ protected:
    virtual int64 seek( int64 pos, Stream::e_whence whence );
 
 public:
-   GenericStream( t_streamType streamType, FileSysData *fsdata ):
+   BaseFileStream( t_streamType streamType, FileSysData *fsdata ):
       Stream( streamType ),
       m_fsData( fsdata )
    {}
 
-   GenericStream( const GenericStream &other );
+   BaseFileStream( const BaseFileStream &other );
 
-   ~GenericStream()
-   {
-      close();
-      delete m_fsData;
-   }
+   virtual ~BaseFileStream();
 
    virtual bool close();
    virtual int32 read( void *buffer, int32 size );
@@ -139,10 +135,10 @@ public:
    */
    void setError( int64 errorCode );
 
-   virtual UserData *clone() const;
+   virtual FalconData *clone() const;
 };
 
-class FALCON_DYN_CLASS FileStream: public GenericStream
+class FALCON_DYN_CLASS FileStream: public BaseFileStream
 {
 public:
    /** Constructs a stream based on system resources.
@@ -150,7 +146,7 @@ public:
       containing the resoruces the target system need to access files.
    */
    FileStream( FileSysData *fsdata ):
-      GenericStream( t_file, fsdata )
+      BaseFileStream( t_file, fsdata )
    {
       status( t_open );
    }
@@ -185,7 +181,7 @@ public:
 
 };
 
-class FALCON_DYN_CLASS StdStream: public GenericStream
+class FALCON_DYN_CLASS StdStream: public BaseFileStream
 {
    virtual int64 seek( int64 pos, Stream::e_whence whence ) {
       m_status = t_unsupported;
@@ -194,7 +190,7 @@ class FALCON_DYN_CLASS StdStream: public GenericStream
 
 public:
    StdStream( FileSysData *fsdata ):
-      GenericStream( t_stream, fsdata )
+      BaseFileStream( t_stream, fsdata )
    {
       status( t_open );
    }
@@ -211,7 +207,7 @@ public:
       return -1;
    }
 */
-   virtual bool truncate( int pos ) {
+   virtual bool truncate( int64 pos=-1 ) {
       m_status = t_unsupported;
       return false;
    }
@@ -240,7 +236,7 @@ public:
       StdStream( fsdata )
    {}
 
-   virtual int32 read( const void *buffer, int32 size ) {
+   virtual int32 read( void *buffer, int32 size ) {
       m_status = t_unsupported;
       return -1;
    }
@@ -348,19 +344,19 @@ public:
    RawStdErrStream();
 };
 
-inline GenericStream::t_attributes operator|(  GenericStream::t_attributes one, GenericStream::t_attributes two)
+inline BaseFileStream::t_attributes operator|(  BaseFileStream::t_attributes one, BaseFileStream::t_attributes two)
 {
-   return (GenericStream::t_attributes) ( ((uint32)one) | ((uint32)two) );
+   return (BaseFileStream::t_attributes) ( ((uint32)one) | ((uint32)two) );
 }
 
-inline GenericStream::t_attributes operator&(  GenericStream::t_attributes one, GenericStream::t_attributes two)
+inline BaseFileStream::t_attributes operator&(  BaseFileStream::t_attributes one, BaseFileStream::t_attributes two)
 {
-   return (GenericStream::t_attributes) ( ((uint32)one) & ((uint32)two) );
+   return (BaseFileStream::t_attributes) ( ((uint32)one) & ((uint32)two) );
 }
 
-inline GenericStream::t_attributes operator^(  GenericStream::t_attributes one, GenericStream::t_attributes two)
+inline BaseFileStream::t_attributes operator^(  BaseFileStream::t_attributes one, BaseFileStream::t_attributes two)
 {
-   return (GenericStream::t_attributes) ( ((uint32)one) & ((uint32)two) );
+   return (BaseFileStream::t_attributes) ( ((uint32)one) & ((uint32)two) );
 }
 
 }
