@@ -21,39 +21,40 @@
 #define flc_flc_itemid_H
 
 
-#define FLC_ITEM_NIL       0
-#define FLC_ITEM_INT       1
-#define FLC_ITEM_NUM       2
-#define FLC_ITEM_RANGE     3
-#define FLC_ITEM_BOOL      4
-
-#define FLC_ITEM_FUNC      6
-/* Some shallow callable items. */
-#define FLC_ITEM_FBOM      7
-
-/** Special VM item */
-#define FLC_ITEM_ATTRIBUTE 8
-/** Used to store pointers in temporary local items by
-   two-step VM functions. */
-#define FLC_ITEM_POINTER   9
+#define FLC_ITEM_NIL          0
+#define FLC_ITEM_BOOL         1
+#define FLC_ITEM_INT          2
+#define FLC_ITEM_NUM          3
 
 /** From this point on, we have possibly deep items */
-#define FLC_ITEM_FIRST_DEEP 10
+#define FLC_ITEM_FIRST_DEEP   4
+#define FLC_ITEM_RANGE        4
 
-#define FLC_ITEM_STRING    10
-#define FLC_ITEM_ARRAY     11
-#define FLC_ITEM_DICT      12
-#define FLC_ITEM_OBJECT    13
-#define FLC_ITEM_MEMBUF    14
+#define FLC_ITEM_LBIND        5
+#define FLC_ITEM_FUNC         6
+/* Some shallow callable items. */
 
-#define FLC_ITEM_REFERENCE 15
-
-#define FLC_ITEM_CLSMETHOD 20
-#define FLC_ITEM_METHOD    22
-#define FLC_ITEM_CLASS     24
+/** Used to store pointers in temporary local items by
+   two-step VM functions. */
+#define FLC_ITEM_GCPTR        7
 
 
-#define FLC_ITEM_INVALID   99
+#define FLC_ITEM_STRING       8
+#define FLC_ITEM_ARRAY        9
+#define FLC_ITEM_DICT         10
+#define FLC_ITEM_OBJECT       11
+#define FLC_ITEM_MEMBUF       12
+
+#define FLC_ITEM_REFERENCE    13
+
+
+#define FLC_ITEM_CLSMETHOD    14
+#define FLC_ITEM_METHOD       15
+#define FLC_ITEM_CLASS        16
+
+#define FLC_ITEM_COUNT        17
+
+#define FLC_ITEM_INVALID      99
 
 #endif
 
