@@ -25,7 +25,7 @@
 
 #include <falcon/setup.h>
 #include <falcon/types.h>
-#include <falcon/userdata.h>
+#include <falcon/falcondata.h>
 #include <falcon/string.h>
 #include <falcon/vm_sys.h>
 
@@ -47,7 +47,7 @@ namespace Falcon {
    system-specific or system independent implementations.
 */
 
-class FALCON_DYN_CLASS Stream: public UserData
+class FALCON_DYN_CLASS Stream: public FalconData
 {
 protected:
    uint32 *m_rhBuffer;
@@ -102,7 +102,7 @@ public:
       t_error = 0x4,
       t_unsupported = 0x8,
       t_invalid = 0x10,
-      t_interrupted = 0x20,
+      t_interrupted = 0x20
 
    } t_status ;
 
@@ -114,12 +114,12 @@ protected:
 
    /** Initializes the base file class. */
    Stream( t_streamType streamType ):
-      m_lastMoved( 0 ),
-      m_status( t_none ),
       m_rhBuffer( 0 ),
       m_rhBufferSize( 0 ),
       m_rhBufferPos( 0 ),
-      m_streamType( streamType )
+      m_streamType( streamType ),
+      m_status( t_none ),
+      m_lastMoved( 0 )
    {}
 
    typedef enum {
@@ -128,7 +128,6 @@ protected:
       ew_end
    } e_whence;
 
-   virtual int64 seek( int64 pos, e_whence w );
    friend class Transcoder;
 
 public:
@@ -153,6 +152,10 @@ public:
 
    virtual ~Stream();
 
+   virtual void gcMark( uint32 mark ) {}
+   virtual bool isStreamBuffer() const { return false; }
+   virtual bool isTranscoder() const { return false; }
+
    /** Reads from target stream.
 
       \param buffer the buffer where read data will be stored.
@@ -192,6 +195,8 @@ public:
    int64 seekEnd( int64 pos ) {
       return seek( pos, ew_end );
    }
+   
+   virtual int64 seek( int64 pos, e_whence w );
 
    virtual int64 lastError() const;
 
@@ -318,7 +323,7 @@ public:
       This version returns 0 and sets error to unsupported;
       subclasses must properly clone the stream.
    */
-   virtual UserData *clone() const;
+   virtual FalconData *clone() const;
 };
 
 
