@@ -30,6 +30,10 @@
 #include "confparser_mod.h"
 #include "confparser_st.h"
 
+/*#
+   @beginmodule feather_configparser
+*/
+
 namespace Falcon {
 namespace Ext {
 
@@ -43,7 +47,7 @@ namespace Ext {
    @optparam filename The name of the ini file to be parsed, if it exists.
    @optparam encoding An optional non default encoding which is used for that file.
 
-   @section Adding, setting or removing keys
+   @section confparser_key_manage Adding, setting or removing keys
 
    The most direct way to add a new key in an ini configuration file is to use the
    @a ConfParser.add method.
@@ -76,7 +80,7 @@ namespace Ext {
    be used to remove the main section; in fact, even if empty, that section always
    exists. To clear every key in that, use the clearMain() method.
 
-   @section Categorized keys
+   @section confparser_categorized_keys Categorized keys
 
    Categories are separated from the keys by dots "."; a complete categorized key
    may contain any amount of dots, or in other words, the categories can have an
@@ -91,7 +95,7 @@ namespace Ext {
    category only, or it can be arbitrarily deep. Only the keys in deeper categories
    will be returned.
 
-   In example; if the configuration file contains the following entries:
+   For example; if the configuration file contains the following entries:
 
    @code
    Key.cat1 = 1
@@ -141,7 +145,7 @@ namespace Ext {
    ordinary get() or getOne() methods, a whole category tree can be imported with
    the method getCategory().
 
-   In example, consider the same configuration structure we have used before. If
+   For example, consider the same configuration structure we have used before. If
    the category parameter  of getCategory() is set to "Key", all the entries will
    be returned. If it's set to "cat1", the first entry won't be returned, as it's
    consider a key cat1 in category Key. If category is set to "key.cat1.subcat1",
@@ -152,7 +156,7 @@ namespace Ext {
    stripped of their category part by adding an asterisk at the end of the first
    parameter.
 
-   In example:
+   For example:
 
    @code
    category = "Key.cat1"
@@ -181,6 +185,8 @@ namespace Ext {
 
 /*#
    @init ConfParser
+   
+   @brief Prepares the confparser, and eventually opens a config file.
 
    The constructor of this class allows to set up a filename for the
    configuration file that is going to be read and/or written. If the name is not
@@ -196,7 +202,7 @@ FALCON_FUNC  ConfParser_init( ::Falcon::VMachine *vm )
 
    if ( (i_fname != 0 && ! i_fname->isString()) || ( i_encoding != 0 && ! i_encoding->isString() ) )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, [S]" ) ) );
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, [S]" ) );
       return;
    }
 
@@ -225,7 +231,7 @@ FALCON_FUNC  ConfParser_init( ::Falcon::VMachine *vm )
    readable Falcon stream. If not, the file name provided to the ConfParser
    constructor will be opened and read. In case the name has not been given in the
    constructor, the method raises an error. The method may also raise ParseError,
-   IoError or ParamError, with the “message” field set to a relevant explanation.
+   IoError or ParamError, with the "message" field set to a relevant explanation.
 */
 
 FALCON_FUNC  ConfParser_read( ::Falcon::VMachine *vm )
@@ -238,7 +244,9 @@ FALCON_FUNC  ConfParser_read( ::Falcon::VMachine *vm )
 
    if( i_stream == 0 )
    {
+      vm->idle();
       bRes = cfile->load();
+      vm->unidle();
    }
    else {
       bool bValid = false;
@@ -255,7 +263,7 @@ FALCON_FUNC  ConfParser_read( ::Falcon::VMachine *vm )
 
       if ( ! bValid )
       {
-         vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "Stream" ) ) );
+         throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "Stream" ) );
          return;
       }
    }
@@ -265,18 +273,18 @@ FALCON_FUNC  ConfParser_read( ::Falcon::VMachine *vm )
       // is this an I/O or a parsing error?
       if ( cfile->fsError() != 0 )
       {
-         vm->raiseModError( new IoError( ErrorParam( e_loaderror, __LINE__ ).
+         throw new IoError( ErrorParam( e_loaderror, __LINE__ ).
             sysError( cfile->fsError() ).
-            extra( cfile->errorMessage() ) ) );
+            extra( cfile->errorMessage() ) );
       }
       else {
          String msg = cfile->errorMessage() + " at ";
          msg.writeNumber( (int64) cfile->errorLine() );
-         vm->raiseModError( new ParseError( ErrorParam( FALCP_ERR_INVFORMAT, __LINE__ )
-            .desc( FAL_STR(cp_msg_invformat) )
-            .extra( msg ) ) );
          self->setProperty( "error", cfile->errorMessage() );
          self->setProperty( "errorLine", (int64) cfile->errorLine() );
+         throw new ParseError( ErrorParam( FALCP_ERR_INVFORMAT, __LINE__ )
+            .desc( FAL_STR(cp_msg_invformat) )
+            .extra( msg ) );
       }
    }
 
@@ -323,7 +331,7 @@ FALCON_FUNC  ConfParser_write( ::Falcon::VMachine *vm )
 
       if ( ! bValid )
       {
-         vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "Stream" ) ) );
+         throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "Stream" ) );
          return;
       }
    }
@@ -333,17 +341,17 @@ FALCON_FUNC  ConfParser_write( ::Falcon::VMachine *vm )
       // is this a file error?
       if ( cfile->fsError() )
       {
-         vm->raiseModError( new IoError( ErrorParam( e_file_output, __LINE__ ).
+         throw new IoError( ErrorParam( e_file_output, __LINE__ ).
             sysError( cfile->fsError() ).
-            extra( cfile->errorMessage() ) ) );
+            extra( cfile->errorMessage() ) );
       }
       else
       {
          // no -- it's a configuration file.d
-         vm->raiseModError( new ParseError( ErrorParam( FALCP_ERR_STORE, __LINE__ ).
-            desc( FAL_STR(cp_msg_errstore)  ).extra( cfile->errorMessage() ) ) );
          self->setProperty( "error", cfile->errorMessage() );
          self->setProperty( "errorLine", (int64) cfile->errorLine() );
+         throw new ParseError( ErrorParam( FALCP_ERR_STORE, __LINE__ ).
+            desc( FAL_STR(cp_msg_errstore)  ).extra( cfile->errorMessage() ) );
       }
    }
 }
@@ -381,8 +389,7 @@ FALCON_FUNC  ConfParser_get( ::Falcon::VMachine *vm )
         ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
    }
 
    String value;
@@ -407,12 +414,12 @@ FALCON_FUNC  ConfParser_get( ::Falcon::VMachine *vm )
    String value1;
    if ( cfile->getNextValue( value1 ) )
    {
-      CoreArray *array = new CoreArray( vm, 5 );
-      array->append( new GarbageString( vm, value ) );
-      array->append( new GarbageString( vm, value1 ) );
+      CoreArray *array = new CoreArray( 5 );
+      array->append( new CoreString( value ) );
+      array->append( new CoreString( value1 ) );
 
       while( cfile->getNextValue( value1 ) )
-         array->append( new GarbageString( vm, value1 ) );
+         array->append( new CoreString( value1 ) );
 
       vm->retval( array );
    }
@@ -443,8 +450,7 @@ FALCON_FUNC  ConfParser_getOne( ::Falcon::VMachine *vm )
         ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
    }
 
    String value;
@@ -490,8 +496,7 @@ FALCON_FUNC  ConfParser_getMultiple( ::Falcon::VMachine *vm )
         ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
    }
 
    String value;
@@ -511,12 +516,12 @@ FALCON_FUNC  ConfParser_getMultiple( ::Falcon::VMachine *vm )
       }
    }
 
-   CoreArray *array = new CoreArray( vm, 5 );
-   array->append( new GarbageString( vm, value ) );
+   CoreArray *array = new CoreArray( 5 );
+   array->append( new CoreString( value ) );
 
    String value1;
    while( cfile->getNextValue( value1 ) )
-      array->append( new GarbageString( vm, value1 ) );
+      array->append( new CoreString( value1 ) );
 
    vm->retval( array );
 }
@@ -535,13 +540,13 @@ FALCON_FUNC  ConfParser_getSections( ::Falcon::VMachine *vm )
    ConfigFile *cfile = (ConfigFile *) self->getUserData();
 
    String section;
-   CoreArray *ret = new CoreArray( vm );
+   CoreArray *ret = new CoreArray;
 
    if( cfile->getFirstSection( section ) )
    {
-      ret->append( new GarbageString( vm, section ) );
+      ret->append( new CoreString( section ) );
       while( cfile->getNextSection( section ) )
-         ret->append( new GarbageString( vm, section ) );
+         ret->append( new CoreString( section ) );
    }
 
    vm->retval( ret );
@@ -568,12 +573,12 @@ FALCON_FUNC  ConfParser_getKeys( ::Falcon::VMachine *vm )
 
    if ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
       return;
    }
 
    String key;
-   CoreArray *ret = new CoreArray( vm );
+   CoreArray *ret = new CoreArray;
    bool next;
 
    if ( i_section != 0 && ! i_section->isNil() ) {
@@ -585,7 +590,7 @@ FALCON_FUNC  ConfParser_getKeys( ::Falcon::VMachine *vm )
 
    while ( next )
    {
-      ret->append( new GarbageString( vm, key ) );
+      ret->append( new CoreString( key ) );
       next = cfile->getNextKey( key );
    }
 
@@ -614,12 +619,12 @@ FALCON_FUNC  ConfParser_getCategoryKeys( ::Falcon::VMachine *vm )
         ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
       return;
    }
 
    String key;
-   CoreArray *ret = new CoreArray( vm );
+   CoreArray *ret = new CoreArray;
    bool next;
 
    if ( i_section != 0 && ! i_section->isNil() ) {
@@ -631,7 +636,7 @@ FALCON_FUNC  ConfParser_getCategoryKeys( ::Falcon::VMachine *vm )
 
    while ( next )
    {
-      ret->append( new GarbageString( vm, String( key, i_keyMask->asString()->length() + 1 ) ) );
+      ret->append( new CoreString( String( key, i_keyMask->asString()->length() + 1 ) ) );
       next = cfile->getNextKey( key );
    }
 
@@ -662,16 +667,15 @@ FALCON_FUNC  ConfParser_getCategory( ::Falcon::VMachine *vm )
         ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
    }
 
    if ( i_section != 0 && i_section->isNil() )
       i_section = 0;
 
    String key, keymask;
-   CoreDict *ret = new LinearDict( vm );
-   CoreDict *current = ret;
+   LinearDict *ret = new LinearDict();
+   LinearDict *current = ret;
    bool next;
 
    bool stripNames;
@@ -709,30 +713,30 @@ FALCON_FUNC  ConfParser_getCategory( ::Falcon::VMachine *vm )
       String value1;
       if ( cfile->getNextValue( value1 ) )
       {
-         CoreArray *array = new CoreArray( vm, 5 );
-         array->append( new GarbageString( vm, value ) );
-         array->append( new GarbageString( vm, value1 ) );
+         CoreArray *array = new CoreArray( 5 );
+         array->append( new CoreString( value ) );
+         array->append( new CoreString( value1 ) );
 
          while( cfile->getNextValue( value1 ) )
-            array->append( new GarbageString( vm, value1 ) );
+            array->append( new CoreString( value1 ) );
 
          // we have used KEY; now what we want to save is just the non-category
          if ( stripNames )
-            current->insert( new GarbageString( vm, key, keymask.length() + 1 ), array );
+            current->put( new CoreString( key, keymask.length() + 1 ), array );
          else
-            current->insert( new GarbageString( vm, key), array );
+            current->put( new CoreString( key), array );
       }
       else {
           if ( stripNames )
-            current->insert( new GarbageString( vm, key, keymask.length() + 1 ), new GarbageString( vm, value ) );
+            current->put( new CoreString( key, keymask.length() + 1 ), new CoreString( value ) );
          else
-            current->insert(  new GarbageString( vm, key) , new GarbageString( vm, value ) );
+            current->put(  new CoreString( key) , new CoreString( value ) );
       }
 
       next = cfile->getNextKey( key );
    }
 
-   vm->retval( ret );
+   vm->retval( new CoreDict(ret) );
 }
 
 
@@ -756,13 +760,12 @@ FALCON_FUNC  ConfParser_getDictionary( ::Falcon::VMachine *vm )
 
    if ( i_section != 0 && ! i_section->isString() )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
    }
 
    String key;
-   CoreDict *ret = new LinearDict( vm );
-   CoreDict *current = ret;
+   LinearDict *ret = new LinearDict();
+   LinearDict *current = ret;
    bool next;
 
    if ( i_section != 0 ) {
@@ -786,23 +789,23 @@ FALCON_FUNC  ConfParser_getDictionary( ::Falcon::VMachine *vm )
       String value1;
       if ( cfile->getNextValue( value1 ) )
       {
-         CoreArray *array = new CoreArray( vm, 5 );
-         array->append( new GarbageString( vm, value ) );
-         array->append( new GarbageString( vm, value1 ) );
+         CoreArray *array = new CoreArray( 5 );
+         array->append( new CoreString( value ) );
+         array->append( new CoreString( value1 ) );
 
          while( cfile->getNextValue( value1 ) )
-            array->append( new GarbageString( vm, value1 ) );
+            array->append( new CoreString( value1 ) );
 
-         current->insert( new GarbageString( vm, key ), array );
+         current->put( new CoreString( key ), array );
       }
       else {
-         current->insert( new GarbageString( vm, key ), new GarbageString( vm, value ) );
+         current->put( new CoreString( key ), new CoreString( value ) );
       }
 
       next = cfile->getNextKey( key );
    }
 
-   vm->retval( ret );
+   vm->retval( new CoreDict(ret) );
 }
 
 /*#
@@ -829,8 +832,7 @@ FALCON_FUNC  ConfParser_add( ::Falcon::VMachine *vm )
         ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, S" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, S" ) );
    }
 
    String *value;
@@ -841,7 +843,7 @@ FALCON_FUNC  ConfParser_add( ::Falcon::VMachine *vm )
       value = i_value->asString();
    }
    else {
-      value = new GarbageString( vm );
+      value = new CoreString;
       delValue = true;
       vm->itemToString( *value, i_value );
    }
@@ -877,8 +879,7 @@ FALCON_FUNC  ConfParser_set( ::Falcon::VMachine *vm )
         ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, S" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, S" ) );
    }
 
    if ( i_section != 0 && i_section->isNil() )
@@ -902,7 +903,7 @@ FALCON_FUNC  ConfParser_set( ::Falcon::VMachine *vm )
             value = itm.asString();
          }
          else {
-            value = new GarbageString( vm );
+            value = new CoreString;
             delValue = true;
             vm->itemToString( *value, &itm );
          }
@@ -938,7 +939,7 @@ FALCON_FUNC  ConfParser_set( ::Falcon::VMachine *vm )
       value = i_value->asString();
    }
    else {
-      value = new GarbageString( vm );
+      value = new CoreString;
       delValue = true;
       vm->itemToString( *value, i_value );
    }
@@ -977,8 +978,7 @@ FALCON_FUNC  ConfParser_remove( ::Falcon::VMachine *vm )
          ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, S" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, S" ) );
    }
 
    if ( i_section == 0 || i_section->isNil() )
@@ -1013,8 +1013,7 @@ FALCON_FUNC  ConfParser_removeCategory( ::Falcon::VMachine *vm )
          ( i_section != 0 && ! i_section->isString() && ! i_section->isNil() )
       )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, S" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S, S" ) );
    }
 
    if ( i_section == 0 || i_section->isNil() )
@@ -1045,8 +1044,7 @@ FALCON_FUNC  ConfParser_addSection( ::Falcon::VMachine *vm )
 
    if ( i_section == 0 ||  ! i_section->isString() )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S" ) );
    }
 
    vm->retval( (int64) ( cfile->addSection( *i_section->asString() ) == 0 ? 0: 1) );
@@ -1073,8 +1071,7 @@ FALCON_FUNC  ConfParser_removeSection( ::Falcon::VMachine *vm )
 
    if ( i_section == 0 ||  ! i_section->isString() )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).extra( "S" ) );
    }
 
    vm->retval( (int64) ( cfile->removeSection( *i_section->asString() ) ? 1: 0) );
