@@ -61,8 +61,8 @@ static MXML::Node *internal_getNodeParameter( VMachine *vm, int pid )
 
    if ( i_child == 0 || ! i_child->isObject() || ! i_child->asObject()->derivedFrom( "MXMLNode" ) )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "MXMLNode" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "MXMLNode" ) );
       return 0;
    }
 
@@ -81,7 +81,7 @@ static MXML::Node *internal_getNodeParameter( VMachine *vm, int pid )
    an XML document must still be valid as an XML element of another document,
    but it implements some data specific for handling documents.
 
-   @section MXML document structure.
+   @section mxml_doc_struct MXML document structure.
 
    The XML document, as seen by the MXML module, is a tree of nodes. Some nodes have
    meta-informative value, and are meant to be used by the XML parser programs to
@@ -172,8 +172,8 @@ FALCON_FUNC MXMLDocument_init( ::Falcon::VMachine *vm )
    if ( ( i_encoding != 0 && ! i_encoding->isString() && ! i_encoding->isNil() ) ||
       ( i_style != 0 && ! i_style->isInteger()) )
    {
-       vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "[S,I]" ) ) );
+       throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "[S,I]" ) );
       return;
    }
 
@@ -206,8 +206,8 @@ FALCON_FUNC MXMLDocument_deserialize( ::Falcon::VMachine *vm )
 
    if ( i_stream == 0 || ! i_stream->isObject() || ! i_stream->asObject()->derivedFrom( "Stream" ) )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "Stream" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "Stream" ) );
       return;
    }
 
@@ -221,15 +221,15 @@ FALCON_FUNC MXMLDocument_deserialize( ::Falcon::VMachine *vm )
    }
    catch( MXML::MalformedError &err )
    {
-      vm->raiseModError( new MXMLError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
+      throw new MXMLError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
       .desc( err.description() )
-      .extra( err.describeLine() ) ) );
+      .extra( err.describeLine() ) );
    }
    catch( MXML::IOError &err )
    {
-      vm->raiseModError( new IoError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
+      throw new IoError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
       .desc( err.description() )
-      .extra( err.describeLine() ) ) );
+      .extra( err.describeLine() ) );
    }
 }
 
@@ -251,8 +251,8 @@ FALCON_FUNC MXMLDocument_serialize( ::Falcon::VMachine *vm )
 
    if ( i_stream == 0 || ! i_stream->isObject() || ! i_stream->asObject()->derivedFrom( "Stream" ) )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "Stream" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "Stream" ) );
       return;
    }
 
@@ -266,15 +266,15 @@ FALCON_FUNC MXMLDocument_serialize( ::Falcon::VMachine *vm )
    }
    catch( MXML::MalformedError &err )
    {
-      vm->raiseModError( new MXMLError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
+      throw new MXMLError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
       .desc( err.description() )
-      .extra( err.describeLine() ) ) );
+      .extra( err.describeLine() ) );
    }
    catch( MXML::IOError &err )
    {
-      vm->raiseModError( new IoError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
+      throw new IoError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
       .desc( err.description() )
-      .extra( err.describeLine() ) ) );
+      .extra( err.describeLine() ) );
    }
 }
 
@@ -310,8 +310,8 @@ FALCON_FUNC MXMLDocument_style( ::Falcon::VMachine *vm )
 
    if ( i_style == 0 || ! i_style->isInteger() )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "N" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "N" ) );
       return;
    }
 
@@ -437,8 +437,8 @@ FALCON_FUNC MXMLDocument_find( ::Falcon::VMachine *vm )
        ( i_data != 0 && (! i_data->isString() && ! i_data->isNil() ))
    )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S,[S,S,S]" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S,[S,S,S]" ) );
       return;
    }
 
@@ -535,8 +535,8 @@ Item *i_name = vm->param(0);
    // parameter sanity check
    if( i_name == 0 || ! i_name->isString() )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S" ) );
       return;
    }
 
@@ -593,17 +593,18 @@ FALCON_FUNC MXMLDocument_save( ::Falcon::VMachine *vm )
 
    if ( i_uri == 0 || ! i_uri->isString() )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "N" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "N" ) );
       return;
    }
 
    String &uri = *i_uri->asString();
    MXML::Document *doc = static_cast<MXML::DocumentCarrier *>( self->getUserData() )->document();
-
+   
+   vm->idle();
    //TODO: use parsing uri
    FileStream out;
-   if ( out.create( uri, GenericStream::e_aUserRead | GenericStream::e_aUserWrite | GenericStream::e_aGroupRead | GenericStream::e_aOtherRead  ) )
+   if ( out.create( uri, BaseFileStream::e_aUserRead | BaseFileStream::e_aUserWrite | BaseFileStream::e_aGroupRead | BaseFileStream::e_aOtherRead  ) )
    {
 
       Stream *output = &out;
@@ -613,8 +614,9 @@ FALCON_FUNC MXMLDocument_save( ::Falcon::VMachine *vm )
          output = TranscoderFactory( doc->encoding(), &out, false );
          if ( output == 0 )
          {
-            vm->raiseModError( new MXMLError( ErrorParam( e_inv_params, __LINE__ )
-               .extra( "Invalid encoding " + doc->encoding() ) ) );
+            vm->unidle();
+            throw new MXMLError( ErrorParam( e_inv_params, __LINE__ )
+               .extra( "Invalid encoding " + doc->encoding() ) );
             return;
          }
       }
@@ -622,29 +624,31 @@ FALCON_FUNC MXMLDocument_save( ::Falcon::VMachine *vm )
       try
       {
          doc->write( *output, doc->style() );
+         vm->unidle();
          vm->retval( true );
       }
       catch( MXML::MalformedError &err )
       {
-         vm->raiseModError( new MXMLError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
-         .desc( err.description() )
-         .extra( err.describeLine() ) ) );
+         vm->unidle();
+         throw new MXMLError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
+            .desc( err.description() )
+            .extra( err.describeLine() ) );
       }
       catch( MXML::IOError &err )
       {
-         vm->raiseModError( new IoError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
+         vm->unidle();
+         throw new IoError( ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
          .desc( err.description() )
-         .extra( err.describeLine() ) ) );
+         .extra( err.describeLine() ) );
       }
    }
    else
    {
-      vm->raiseModError( new IoError( ErrorParam(
+      vm->unidle();
+      throw new IoError( ErrorParam(
          FALCON_MXML_ERROR_BASE + (int) MXML::Error::errIo , __LINE__ )
-         .desc( FAL_STR( MXML_ERR_IO ) ) ) );
+         .desc( FAL_STR( MXML_ERR_IO ) ) );
    }
-
-   out.close();
 }
 
 
@@ -669,14 +673,16 @@ FALCON_FUNC MXMLDocument_load( ::Falcon::VMachine *vm )
 
    if ( i_uri == 0 || ! i_uri->isString() )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S" ) );
       return;
    }
 
    String &uri = *i_uri->asString();
    MXML::Document *doc = static_cast<MXML::DocumentCarrier *>( self->getUserData() )->document();
-
+   
+   vm->idle();
+   
    //TODO: use parsing uri
    FileStream in;
    if ( in.open( uri ) )
@@ -688,8 +694,9 @@ FALCON_FUNC MXMLDocument_load( ::Falcon::VMachine *vm )
          input = TranscoderFactory( doc->encoding(), &in, false );
          if ( input == 0 )
          {
-            vm->raiseModError( new MXMLError( ErrorParam( e_inv_params, __LINE__ )
-            .extra( FAL_STR( MXML_ERR_INVENC ) + doc->encoding() ) ) );
+            vm->unidle();
+            throw new MXMLError( ErrorParam( e_inv_params, __LINE__ )
+               .extra( FAL_STR( MXML_ERR_INVENC ) + doc->encoding() ) );
             return;
          }
 
@@ -698,32 +705,35 @@ FALCON_FUNC MXMLDocument_load( ::Falcon::VMachine *vm )
       try
       {
          doc->read( *input );
+         vm->unidle();
          vm->retval( true );
       }
       catch( MXML::MalformedError &err )
       {
-         vm->raiseModError( new MXMLError(
+         vm->unidle();
+         throw new MXMLError(
             ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
             .desc( err.description() )
-            .extra( err.describeLine() ) ) );
+            .extra( err.describeLine() ) );
       }
       catch( MXML::IOError &err )
       {
-         vm->raiseModError( new IoError(
+         vm->unidle();
+         throw new IoError(
             ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
             .desc( err.description() )
-            .extra( err.describeLine() ) ) );
+            .extra( err.describeLine() ) );
       }
 
       in.close();
       return;
    }
-
+   
    if ( ! in.good() )
    {
-      vm->raiseModError( new IoError( ErrorParam(
+      throw new IoError( ErrorParam(
          FALCON_MXML_ERROR_BASE + (int) MXML::Error::errIo , __LINE__ )
-         .desc( FAL_STR( MXML_ERR_IO )  ) ) );
+         .desc( FAL_STR( MXML_ERR_IO ) ) );
    }
 
    in.close();
@@ -749,8 +759,8 @@ FALCON_FUNC MXMLDocument_setEncoding( ::Falcon::VMachine *vm )
 
    if ( i_encoding == 0 || ! i_encoding->isString() )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S" ) );
       return;
    }
 
@@ -758,8 +768,8 @@ FALCON_FUNC MXMLDocument_setEncoding( ::Falcon::VMachine *vm )
    Transcoder *tr = TranscoderFactory( encoding );
    if ( tr == 0 )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_param_range, __LINE__ ).
-         extra( encoding ) ) );
+      throw new  ParamError( ErrorParam( e_param_range, __LINE__ ).
+         extra( encoding ) );
       return;
    }
    delete tr;
@@ -789,7 +799,7 @@ FALCON_FUNC MXMLDocument_getEncoding( ::Falcon::VMachine *vm )
 {
    CoreObject *self = vm->self().asObject();
    MXML::Document *doc = static_cast<MXML::DocumentCarrier *>( self->getUserData() )->document();
-   vm->retval( new GarbageString( vm, doc->encoding() ) );
+   vm->retval( new CoreString( doc->encoding() ) );
 }
 
 /*#
@@ -837,8 +847,8 @@ FALCON_FUNC MXMLNode_init( ::Falcon::VMachine *vm )
       ( i_name != 0 && ! (i_name->isString() || i_name->isNil()) ) ||
       ( i_data != 0 && ! i_data->isString() )  )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "[N,S,S]" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "[N,S,S]" ) );
       return;
    }
 
@@ -847,8 +857,8 @@ FALCON_FUNC MXMLNode_init( ::Falcon::VMachine *vm )
 
    if ( type < 0 || type > (int) MXML::Node::typeFakeClosing )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "Invalid type" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "Invalid type" ) );
       return;
    }
 
@@ -877,8 +887,8 @@ FALCON_FUNC MXMLNode_serialize( ::Falcon::VMachine *vm )
 
    if ( i_stream == 0 || ! i_stream->isObject() || ! i_stream->asObject()->derivedFrom( "Stream" ) )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "Stream" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "Stream" ) );
       return;
    }
 
@@ -892,17 +902,17 @@ FALCON_FUNC MXMLNode_serialize( ::Falcon::VMachine *vm )
    }
    catch( MXML::MalformedError &err )
    {
-      vm->raiseModError( new MXMLError(
+      throw new MXMLError(
       ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
          .desc( err.description() )
-         .extra( err.describeLine() ) ) );
+         .extra( err.describeLine() ) );
    }
    catch( MXML::IOError &err )
    {
-      vm->raiseModError( new MXMLError(
+      throw new MXMLError(
          ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
          .desc( err.description() )
-         .extra( err.describeLine() ) ) );
+         .extra( err.describeLine() ) );
    }
 
 }
@@ -924,8 +934,8 @@ FALCON_FUNC MXMLNode_deserialize( ::Falcon::VMachine *vm )
 
    if ( i_stream == 0 || ! i_stream->isObject() || ! i_stream->asObject()->derivedFrom( "Stream" ) )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "Stream" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "Stream" ) );
       return;
    }
 
@@ -942,18 +952,18 @@ FALCON_FUNC MXMLNode_deserialize( ::Falcon::VMachine *vm )
    }
    catch( MXML::MalformedError &err )
    {
-      vm->raiseModError( new MXMLError(
+      throw new MXMLError(
          ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
          .desc( err.description() )
-         .extra( err.describeLine() ) ) );
+         .extra( err.describeLine() ) );
       delete node;
    }
    catch( MXML::IOError &err )
    {
-      vm->raiseModError( new MXMLError(
+      throw new MXMLError(
          ErrorParam( FALCON_MXML_ERROR_BASE + err.numericCode(), __LINE__ )
          .desc( err.description() )
-         .extra( err.describeLine() ) ) );
+         .extra( err.describeLine() ) );
       delete node;
    }
 }
@@ -987,8 +997,8 @@ FALCON_FUNC MXMLNode_name( ::Falcon::VMachine *vm )
 
    if ( i_name != 0 && ! i_name->isString() )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "[S]" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "[S]" ) );
       return;
    }
 
@@ -996,7 +1006,7 @@ FALCON_FUNC MXMLNode_name( ::Falcon::VMachine *vm )
    MXML::Node *node = static_cast<NodeCarrier *>( self->getUserData() )->node();
 
    if ( i_name == 0 )
-      vm->retval( new GarbageString( vm, node->name() ) );
+      vm->retval( new CoreString( node->name() ) );
    else
       node->name( *i_name->asString() );
 }
@@ -1019,15 +1029,15 @@ FALCON_FUNC MXMLNode_data( ::Falcon::VMachine *vm )
 
    if ( i_data != 0 && ! i_data->isString() )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "[S]" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "[S]" ) );
       return;
    }
 
    CoreObject *self = vm->self().asObject();
    MXML::Node *node = static_cast<NodeCarrier *>( self->getUserData() )->node();
    if ( i_data == 0 )
-      vm->retval( new GarbageString( vm, node->data() ) );
+      vm->retval( new CoreString( node->data() ) );
    else
       node->data( *i_data->asString() );
 }
@@ -1062,8 +1072,8 @@ FALCON_FUNC MXMLNode_setAttribute( ::Falcon::VMachine *vm )
         i_attrValue == 0
       )
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S,X" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S,X" ) );
       return;
    }
 
@@ -1109,8 +1119,8 @@ FALCON_FUNC MXMLNode_getAttribute( ::Falcon::VMachine *vm )
 
    if ( i_attrName == 0 || ! i_attrName->isString() )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S" ) ) );
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S" ) );
       return;
    }
 
@@ -1121,7 +1131,7 @@ FALCON_FUNC MXMLNode_getAttribute( ::Falcon::VMachine *vm )
    }
 
    const String &val = node->getAttribute( *i_attrName->asString() );
-   vm->retval( new GarbageString( vm, val ) );
+   vm->retval( new CoreString( val ) );
 }
 
 /*#
@@ -1150,17 +1160,17 @@ FALCON_FUNC MXMLNode_getAttribs( ::Falcon::VMachine *vm )
 
    const MXML::AttribList &attribs = node->attribs();
 
-   CoreDict *dict = new LinearDict( vm, attribs.size() );
+   LinearDict *dict = new LinearDict( attribs.size() );
 
    MXML::AttribList::const_iterator iter = attribs.begin();
    while( iter != attribs.end() )
    {
-      dict->insert( new GarbageString( vm, (*iter)->name()),
-         new GarbageString( vm, (*iter)->value()) );
+      dict->put( new CoreString( (*iter)->name()),
+         new CoreString( (*iter)->value()) );
       ++iter;
    }
 
-   vm->retval( dict );
+   vm->retval( new CoreDict(dict) );
 }
 
 /*#
@@ -1183,7 +1193,7 @@ FALCON_FUNC MXMLNode_getChildren( ::Falcon::VMachine *vm )
    CoreObject *self = vm->self().asObject();
    MXML::Node *node = static_cast<NodeCarrier *>( self->getUserData() )->node();
 
-   CoreArray *arr = new CoreArray( vm );
+   CoreArray *arr = new CoreArray;
 
    node = node->child();
    while( node != 0 )
@@ -1552,7 +1562,7 @@ FALCON_FUNC MXMLNode_path( ::Falcon::VMachine *vm )
 {
    CoreObject *self = vm->self().asObject();
    MXML::Node *node = static_cast<NodeCarrier *>( self->getUserData() )->node();
-   GarbageString *gs = new GarbageString( vm, node->path() );
+   CoreString *gs = new CoreString( node->path() );
    gs->bufferize();
    vm->retval( gs );
 }
