@@ -71,7 +71,7 @@ FALCON_FUNC  falcon_processKill( ::Falcon::VMachine *vm )
 
    if ( id == 0 || ! id->isOrdinal() )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
    }
 
    if ( mode == 0 || ! mode->isTrue() )
@@ -139,22 +139,18 @@ FALCON_FUNC  ProcessEnum_next  ( ::Falcon::VMachine *vm )
 {
    CoreObject *self = vm->self().asObject();
    Sys::ProcessEnum *pe = (Sys::ProcessEnum *)self->getUserData();
-   GarbageString *name = new GarbageString( vm );
-   GarbageString *path = new GarbageString( vm );
+   CoreString *name = new CoreString;
+   CoreString *path = new CoreString;
    uint64 pid, ppid;
 
    int64 res = (int64) pe->next( *name, pid, ppid, *path );
 
    if ( res != 1 )
    {
-      vm->memPool()->destroyGarbage( name );
-      vm->memPool()->destroyGarbage( path );
-
       if ( res == -1 )
       {
-         vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_READLIST, __LINE__ )
-            .desc( FAL_STR(proc_msg_errlist) ) ) );
-         return;
+         throw new ProcessError( ErrorParam( FALPROC_ERR_READLIST, __LINE__ )
+            .desc( FAL_STR(proc_msg_errlist) ) );
       }
    }
    else {
@@ -180,8 +176,8 @@ FALCON_FUNC  ProcessEnum_close  ( ::Falcon::VMachine *vm )
    CoreObject *self = vm->self().asObject();
    Sys::ProcessEnum *pe = (Sys::ProcessEnum *)self->getUserData();
    if ( ! pe->close() ) {
-         vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_CLOSELIST, __LINE__ )
-            .desc( FAL_STR( proc_msg_errlist2 ) ) ) );
+         throw new ProcessError( ErrorParam( FALPROC_ERR_CLOSELIST, __LINE__ )
+            .desc( FAL_STR( proc_msg_errlist2 ) ) );
       return;
    }
 }
@@ -200,8 +196,8 @@ FALCON_FUNC  ProcessEnum_close  ( ::Falcon::VMachine *vm )
    is actually executed by passing the command string to the system command shell.
    In this way, it is possible to  execute commands that are parsed by the shell.
 
-   This includes internal commands as “dir” in Windows systems, or small scripts as
-   “for file in $(ls); do touch $file; done” if the system shell is sh.
+   This includes internal commands as "dir" in Windows systems, or small scripts as
+   "for file in $(ls); do touch $file; done" if the system shell is sh.
    However, loading the shell may generate a needless overhead for the most common
    usages of system(). Use systemCall() if there isn't the need to have the system
    shell to parse the command.
@@ -219,7 +215,7 @@ FALCON_FUNC  falcon_system ( ::Falcon::VMachine *vm )
 
    if( sys_req == 0 || ( sys_req->type() != FLC_ITEM_STRING ) )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
+      new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
       return;
    }
 
@@ -234,12 +230,18 @@ FALCON_FUNC  falcon_system ( ::Falcon::VMachine *vm )
    argv[3] = 0;
 
    int retval;
+   vm->idle();
    if( ::Falcon::Sys::spawn( argv, false, background, &retval ) )
+   {
+      vm->unidle();
       vm->retval( retval );
-   else {
-      vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_CREATLIST, __LINE__ )
+   }
+   else 
+   {
+      vm->unidle();
+      new ProcessError( ErrorParam( FALPROC_ERR_CREATLIST, __LINE__ )
          .desc( FAL_STR(proc_msg_errlist3) )
-         .sysError( retval ) ) );
+         .sysError( retval ) );
    }
 }
 
@@ -278,10 +280,11 @@ FALCON_FUNC  falcon_systemCall ( ::Falcon::VMachine *vm )
 
    if( sys_req == 0 || ( sys_req->type() != FLC_ITEM_STRING &&  sys_req->type() != FLC_ITEM_ARRAY ) )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
-      return;
+      new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
    }
 
+   vm->idle();
+   
    bool background = mode == 0 ? false : mode->isTrue();
    String **argv;
 
@@ -293,9 +296,8 @@ FALCON_FUNC  falcon_systemCall ( ::Falcon::VMachine *vm )
       CoreArray *array = sys_req->asArray();
       for( count = 0; count < array->length(); count ++ )
          if ( array->at( count ).type() != FLC_ITEM_STRING ) {
-            vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-               extra( FAL_STR( proc_msg_allstr ) ) ) );
-            return;
+            throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+               extra( FAL_STR( proc_msg_allstr ) ) );
          }
 
       argv = (String **) memAlloc( (array->length()+1) * sizeof( String * ) );
@@ -308,11 +310,25 @@ FALCON_FUNC  falcon_systemCall ( ::Falcon::VMachine *vm )
 
    int retval;
    if( ::Falcon::Sys::spawn( argv, false, background, &retval ) )
+   {
+      vm->unidle();
       vm->retval( retval );
+      
+      if( sys_req->type() == FLC_ITEM_STRING )
+         ::Falcon::Mod::freeArgv( argv );
+      else
+         memFree( argv );
+   }
    else {
-      vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_CREATPROC, __LINE__ )
+      vm->unidle();
+       if( sys_req->type() == FLC_ITEM_STRING )
+         ::Falcon::Mod::freeArgv( argv );
+      else
+         memFree( argv );
+         
+      throw new ProcessError( ErrorParam( FALPROC_ERR_CREATPROC, __LINE__ )
          .desc( FAL_STR( proc_msg_prccreate ) )
-         .sysError( retval ) ) );
+         .sysError( retval ) );
    }
 
    if( sys_req->type() == FLC_ITEM_STRING )
@@ -321,6 +337,71 @@ FALCON_FUNC  falcon_systemCall ( ::Falcon::VMachine *vm )
       memFree( argv );
 }
 
+
+/*#
+   @function pread
+   @brief Executes an external process and waits for its termination.
+   @param command The command to be executed.
+   @param background If given and true, the process runs hidden.
+   @return The full output generated by the rpcess.
+   @raise ProcessError if the process couldn't be created.
+
+   This function launches an external system command and waits until the command
+   execution is terminated, returning the exit code of the child process. All the 
+   
+   for example:
+
+   @code
+   dir_contents = pread( "ls" )
+   > dir_contents
+   @endcode
+
+   If the process cannot be started if it fails to start, an error is raised.
+   
+   \note This function uses the standard system shell to execute the passed
+   commands, so it is possible to pipe applications and redirect streams
+   via the standard "|" and ">" command line characters.
+*/
+FALCON_FUNC  falcon_pread ( ::Falcon::VMachine *vm )
+{
+   Item *sys_req = vm->param(0);
+   Item *mode = vm->param(1);
+
+   if( sys_req == 0 || ( sys_req->type() != FLC_ITEM_STRING &&  sys_req->type() != FLC_ITEM_ARRAY ) )
+   {
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
+   }
+
+   bool background = mode == 0 ? false : mode->isTrue();
+   String *argv[4];
+
+   String shellName( ::Falcon::Sys::shellName() );
+   String shellParam( ::Falcon::Sys::shellParam() );
+   argv[0] = &shellName;
+   argv[1] = &shellParam;
+   argv[2] = sys_req->asString();
+   argv[3] = 0;
+
+   int retval=0;
+   CoreString* gs = new CoreString;
+   if( ::Falcon::Sys::spawn_read( argv, false, background, &retval, gs ) )
+   {
+      if ( retval == 0x7F00 )
+      {
+         throw new ProcessError( ErrorParam( FALPROC_ERR_CREATPROC, __LINE__ )
+            .desc( FAL_STR( proc_msg_prccreate ) )
+            .sysError( 0 ) );
+      }
+      else
+         vm->retval( gs );
+   }
+   else {
+      throw new ProcessError( ErrorParam( FALPROC_ERR_CREATPROC, __LINE__ )
+         .desc( FAL_STR( proc_msg_prccreate ) )
+         .sysError( retval ) );
+   }
+}
+
 /*#
    @function exec
    @brief Launches a process in place of the host process.
@@ -351,8 +432,7 @@ FALCON_FUNC  falcon_exec ( ::Falcon::VMachine *vm )
 
    if( sys_req == 0 || ( sys_req->type() != FLC_ITEM_STRING &&  sys_req->type() != FLC_ITEM_ARRAY ) )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ) );
    }
 
    String **argv;
@@ -365,9 +445,8 @@ FALCON_FUNC  falcon_exec ( ::Falcon::VMachine *vm )
       CoreArray *array = sys_req->asArray();
       for( count = 0; count < array->length(); count ++ )
          if ( array->at( count ).type() != FLC_ITEM_STRING ) {
-            vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-               extra( FAL_STR( proc_msg_allstr ) ) ) );
-            return;
+            throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+               extra( FAL_STR( proc_msg_allstr ) ) );
          }
 
       argv = (String **) memAlloc( (array->length()+1) * sizeof( char * ) );
@@ -383,8 +462,8 @@ FALCON_FUNC  falcon_exec ( ::Falcon::VMachine *vm )
    if( ::Falcon::Sys::spawn( argv, true, false, &retval ) )
       vm->retval( retval );
    else {
-      vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_CREATPROC, __LINE__ ).
-         desc( FAL_STR( proc_msg_prccreate ) ).sysError( retval ) ) );
+      throw new ProcessError( ErrorParam( FALPROC_ERR_CREATPROC, __LINE__ ).
+         desc( FAL_STR( proc_msg_prccreate ) ).sysError( retval ) );
    }
 
    if( sys_req->type() == FLC_ITEM_STRING )
@@ -436,9 +515,8 @@ FALCON_FUNC  Process_init ( ::Falcon::VMachine *vm )
    if( sys_req == 0 || ( ! sys_req->isString() && ! sys_req->isArray() ) ||
       (mode_itm != 0 && ! mode_itm->isOrdinal())  )
    {
-      vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ )
-         .extra( "( S|A, [I] )" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ )
+         .extra( "( S|A, [I] )" ) );
    }
 
    String **argv;
@@ -471,9 +549,8 @@ FALCON_FUNC  Process_init ( ::Falcon::VMachine *vm )
       CoreArray *array = sys_req->asArray();
       for( count = 0; count < array->length(); count ++ )
          if ( array->at( count ).type() != FLC_ITEM_STRING ) {
-            vm->raiseModError( new ParamError( ErrorParam( e_inv_params, __LINE__ )
-               .extra( FAL_STR( proc_msg_allstr ) ) ) );
-            return;
+            throw new ParamError( ErrorParam( e_inv_params, __LINE__ )
+               .extra( FAL_STR( proc_msg_allstr ) ) );
          }
       argv = (String **) memAlloc( (array->length()+1) * sizeof( String * ) );
       for( count = 0; count < array->length(); count ++ )
@@ -493,10 +570,10 @@ FALCON_FUNC  Process_init ( ::Falcon::VMachine *vm )
    if ( handle->lastError() == 0 )
       vm->self().asObject()->setUserData( handle );
    else {
-      vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_CREATPROC, __LINE__ )
-         .desc( FAL_STR(proc_msg_prccreate) )
-         .sysError( handle->lastError() ) ) );
       delete handle;
+      throw new ProcessError( ErrorParam( FALPROC_ERR_CREATPROC, __LINE__ )
+         .desc( FAL_STR(proc_msg_prccreate) )
+         .sysError( handle->lastError() ) );
    }
 
    if ( delArgs )
@@ -533,13 +610,18 @@ FALCON_FUNC  Process_wait ( ::Falcon::VMachine *vm )
 {
    ::Falcon::Sys::ProcessHandle *handle = (::Falcon::Sys::ProcessHandle *)
       vm->self().asObject()->getUserData();
-   if( ! handle->wait( true ) ) {
-      vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_WAIT, __LINE__ )
+   
+   vm->idle();
+   if( ! handle->wait( true ) ) 
+   {
+      vm->unidle();
+      throw new ProcessError( ErrorParam( FALPROC_ERR_WAIT, __LINE__ )
          .desc( FAL_STR( proc_msg_waitfail ) )
-         .sysError( handle->lastError() ) ) );
+         .sysError( handle->lastError() ) );
    }
    else {
       handle->close();
+      vm->unidle();
    }
 }
 
@@ -569,8 +651,8 @@ FALCON_FUNC  Process_terminate ( ::Falcon::VMachine *vm )
    {
       bool sev = severe == 0 ? false : severe->isTrue();
       if( ! handle->terminate( sev ) ) {
-         vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_TERM, __LINE__ ).
-            desc( FAL_STR( proc_msg_termfail ) ).sysError( handle->lastError() ) ) );
+         throw new ProcessError( ErrorParam( FALPROC_ERR_TERM, __LINE__ ).
+            desc( FAL_STR( proc_msg_termfail ) ).sysError( handle->lastError() ) );
       }
    }
 }
@@ -601,18 +683,23 @@ FALCON_FUNC  Process_value ( ::Falcon::VMachine *vm )
 
    bool wait = h_wait == 0 ? false : h_wait->isTrue();
    if ( wait && ! handle->done() ) {
+      vm->idle();
       if( ! handle->wait( true ) ) {
-         vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_WAIT, __LINE__ )
+         handle->close();
+         vm->unidle();
+         throw new ProcessError( ErrorParam( FALPROC_ERR_WAIT, __LINE__ )
             .desc( FAL_STR( proc_msg_waitfail ) )
-            .sysError( handle->lastError() ) ) );
+            .sysError( handle->lastError() ) );
       }
+      else
+         vm->unidle();
    }
    // give a test to see if the process is terminated in the meanwhile
    else if ( ! handle->done() ) {
       if( ! handle->wait( false ) ) {
-         vm->raiseModError( new ProcessError( ErrorParam( FALPROC_ERR_WAIT, __LINE__ )
+         throw new ProcessError( ErrorParam( FALPROC_ERR_WAIT, __LINE__ )
             .desc( FAL_STR( proc_msg_waitfail ) )
-            .sysError( handle->lastError() ) ) );
+            .sysError( handle->lastError() ) );
       }
    }
 
@@ -690,7 +777,7 @@ FALCON_FUNC  Process_getOutput ( ::Falcon::VMachine *vm )
 }
 
 /*#
-   @method Process_getAux Process
+   @method getAux Process
    @brief Returns the process auxiliary output stream.
    @return The child process auxiliary output stream (read-only)
 
@@ -744,7 +831,7 @@ FALCON_FUNC  ProcessError_init ( ::Falcon::VMachine *vm )
 {
    CoreObject *einst = vm->self().asObject();
    if( einst->getUserData() == 0 )
-      einst->setUserData( new Falcon::ErrorCarrier( new ProcessError ) );
+      einst->setUserData( new ProcessError );
 
    ::Falcon::core::Error_init( vm );
 }
