@@ -101,16 +101,16 @@ A minimal example would look like the following:
    @endcode
 
    The parser provides single quoted literal strings, where the backslash has no special
-   meaning, (except for the sequence \' which is turned into a single quote).
+   meaning.
 
    In example:
    @code
-   re = Regex( '^\s*\w' )
+   re = Regex( '^\\s*\\w' )
    @endcode
 
    This would match any word near the beginning of a line, and it is equivalent to:
    @code
-   re = Regex( "^\\s*\\w" )
+   re = Regex( "^\\\\s*\\\\w" )
    @endcode
 
    Of course, the former is more readable and natural when dealing with regular
@@ -119,10 +119,9 @@ A minimal example would look like the following:
    @beginmodule feather_regex
 */
 
-FALCON_MODULE_DECL( const Falcon::EngineData &data )
+FALCON_MODULE_DECL
 {
    #define FALCON_DECLARE_MODULE self
-   data.set();
 
    Falcon::Module *self = new Falcon::Module();
    self->name( "regex" );
@@ -131,9 +130,9 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    self->version( VERSION_MAJOR, VERSION_MINOR, VERSION_REVISION );
 
    // Initialzie pcre -- todo, import data from app.
-	pcre_malloc = Falcon::memAlloc;
+   pcre_malloc = Falcon::memAlloc;
    pcre_free = Falcon::memFree;
-	pcre_stack_malloc = Falcon::memAlloc;
+   pcre_stack_malloc = Falcon::memAlloc;
    pcre_stack_free = Falcon::memFree;
 
    //====================================
@@ -145,16 +144,29 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
    //
    Falcon::Symbol *regex_c = self->addClass( "Regex", Falcon::Ext::Regex_init );
    self->addClassMethod( regex_c, "study", Falcon::Ext::Regex_study );
-   self->addClassMethod( regex_c, "match", Falcon::Ext::Regex_match );
-   self->addClassMethod( regex_c, "grab", Falcon::Ext::Regex_grab );
-   self->addClassMethod( regex_c, "find", Falcon::Ext::Regex_find );
-   self->addClassMethod( regex_c, "findAll", Falcon::Ext::Regex_findAll );
-   self->addClassMethod( regex_c, "findAllOverlapped", Falcon::Ext::Regex_findAllOverlapped );
-   self->addClassMethod( regex_c, "replace", Falcon::Ext::Regex_replace );
-   self->addClassMethod( regex_c, "replaceAll", Falcon::Ext::Regex_replaceAll );
+   self->addClassMethod( regex_c, "match", Falcon::Ext::Regex_match ).asSymbol()->
+      addParam("string");
+   self->addClassMethod( regex_c, "grab", Falcon::Ext::Regex_grab ).asSymbol()->
+      addParam("string");
+   self->addClassMethod( regex_c, "split", Falcon::Ext::Regex_split ).asSymbol()->
+      addParam("string")->addParam("count")->addParam("gettoken");
+   self->addClassMethod( regex_c, "find", Falcon::Ext::Regex_find ).asSymbol()->
+      addParam("string")->addParam("start");
+   self->addClassMethod( regex_c, "findAll", Falcon::Ext::Regex_findAll ).asSymbol()->
+      addParam("string")->addParam("start")->addParam("maxcount");
+   self->addClassMethod( regex_c, "findAllOverlapped", Falcon::Ext::Regex_findAllOverlapped ).asSymbol()->
+      addParam("string")->addParam("start")->addParam("maxcount");
+   self->addClassMethod( regex_c, "replace", Falcon::Ext::Regex_replace ).asSymbol()->
+      addParam("string")->addParam("replacer");
+   self->addClassMethod( regex_c, "replaceAll", Falcon::Ext::Regex_replaceAll ).asSymbol()->
+      addParam("string")->addParam("replacer");
+   self->addClassMethod( regex_c, "subst", Falcon::Ext::Regex_subst ).asSymbol()->
+      addParam("string")->addParam("replacer");
    self->addClassMethod( regex_c, "capturedCount", Falcon::Ext::Regex_capturedCount );
-   self->addClassMethod( regex_c, "captured", Falcon::Ext::Regex_captured );
-   self->addClassMethod( regex_c, "compare", Falcon::Ext::Regex_compare );
+   self->addClassMethod( regex_c, "captured", Falcon::Ext::Regex_captured ).asSymbol()->
+      addParam("count");
+   self->addClassMethod( regex_c, "compare", Falcon::Ext::Regex_compare ).asSymbol()->
+      addParam("string");
    self->addClassMethod( regex_c, "version", Falcon::Ext::Regex_version );
 
    //==================================================
@@ -169,3 +181,4 @@ FALCON_MODULE_DECL( const Falcon::EngineData &data )
 }
 
 /* end of regex.cpp */
+
