@@ -55,14 +55,14 @@ namespace Ext {
 
 FALCON_FUNC  falcon_getHostName( ::Falcon::VMachine *vm )
 {
-   String *s = new GarbageString( vm );
+   String *s = new CoreString;
    if ( ::Falcon::Sys::getHostName( *s ) )
       vm->retval( s );
    else {
       delete s;
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_GENERIC, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_GENERIC, __LINE__ )
          .desc( FAL_STR( sk_msg_generic ) )
-         .sysError( (uint32) errno ) ) );
+         .sysError( (uint32) errno ) );
    }
 }
 
@@ -88,7 +88,7 @@ FALCON_FUNC  falcon_getHostName( ::Falcon::VMachine *vm )
    method will resolve the target address on its own.
 
    This function can be used on the value returned by @a getHostName, or using
-   "localhost" as the @a address parameter, to receive a
+   "localhost" as the @b address parameter, to receive a
    list of the interfaces that are available under the network name of the host the
    VM is running on. This is useful to i.e. bind some services only to one of the
    available interfaces.
@@ -106,27 +106,25 @@ FALCON_FUNC  resolveAddress( ::Falcon::VMachine *vm )
    Item *address = vm->param( 0 );
    if ( address == 0 || ! address->isString() )
    {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S" ) );
    }
 
    Sys::Address addr;
    addr.set( *address->asString() );
    if ( ! addr.resolve() ) {
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_RESOLV, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_RESOLV, __LINE__ )
          .desc( FAL_STR( sk_msg_errresolv ) )
-         .sysError( (uint32) addr.lastError() ) ) );
-      return;
+         .sysError( (uint32) addr.lastError() ) );
    }
 
-   CoreArray *ret = new CoreArray( vm, addr.getResolvedCount() );
+   CoreArray *ret = new CoreArray( addr.getResolvedCount() );
    String dummy;
    int32 port;
 
    for ( int i = 0; i < addr.getResolvedCount(); i ++ )
    {
-      String *s = new GarbageString( vm );
+      String *s = new CoreString;
       addr.getResolvedEntry( i, *s, dummy, port );
       ret->append( s );
    }
@@ -154,7 +152,7 @@ FALCON_FUNC  socketErrorDesc( ::Falcon::VMachine *vm )
       vm->retnil();
    }
    else {
-      String *str = new GarbageString( vm );
+      String *str = new CoreString;
       if ( Sys::getErrorDesc( code->asInteger(), *str ) )
          vm->retval( str );
       else
@@ -169,15 +167,16 @@ FALCON_FUNC  socketErrorDesc( ::Falcon::VMachine *vm )
 
 /*#
    @class Socket
-   @brief TCP/IP networking base class,
+   @brief TCP/IP networking base class.
+   
    The Socket class is the base class for both UDP and TCP socket.
    It provides common methods and properties,
    and so it should not be directly instantiated.
 
-   @prop timedout True if the last operation has timed out. See @a Socket.setTimeout..
+   @prop timedout True if the last operation has timed out. See @a Socket.setTimeout.
 
    @prop lastError Numeric value of system level error that has occoured on the socket.
-      @a getErrorDescription may be used to get a human-readable description of the error.
+      Standard Function @b socketErrorDesc may be used to get a human-readable description of the error.
       The error is usually also written in the fsError field of the exceptions,
       if case they are caught.
 */
@@ -191,6 +190,7 @@ FALCON_FUNC  Socket_init( ::Falcon::VMachine *vm )
 
 /*#
    @method setTimeout Socket
+   @brief Sets the default timeout for lengthy operations.
    @param timeout Timeout in seconds and fractions.
 
    This function sets a default timeout for the read/write operations, or for other
@@ -216,9 +216,8 @@ FALCON_FUNC  Socket_setTimeout( ::Falcon::VMachine *vm )
    Item *i_to = vm->param(0);
    if ( i_to == 0 || ! i_to->isOrdinal() )
    {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "N" ) ) );
-      return;
+      throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "N" ) );
    }
 
    CoreObject *self = vm->self().asObject();
@@ -294,9 +293,8 @@ FALCON_FUNC  Socket_readAvailable( ::Falcon::VMachine *vm )
    Item *to = vm->param( 0 );
    if ( to != 0 && ! to->isOrdinal() )
    {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "[N]" ) ) );
-      return;
+      throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "[N]" ) );
    }
 
    int64 timeout = to == 0 ? -1 : int64(to->forceNumeric() * 1000.0);
@@ -305,8 +303,11 @@ FALCON_FUNC  Socket_readAvailable( ::Falcon::VMachine *vm )
    Sys::Socket *tcps = (Sys::Socket *) self->getUserData();
    int res;
 
+   if ( timeout > 0 ) vm->idle();
    if ( (res = tcps->readAvailable( (int32)timeout, &vm->systemData() ) ) <= 0 )
    {
+      if ( timeout > 0 ) vm->unidle();
+      
       // interrupted?
       if ( res == -2 )
       {
@@ -319,14 +320,16 @@ FALCON_FUNC  Socket_readAvailable( ::Falcon::VMachine *vm )
          vm->regA().setBoolean( false );
       }
       else {
-         vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_GENERIC, __LINE__ )
-            .desc( FAL_STR( sk_msg_generic ) )
-            .sysError( (uint32) tcps->lastError() ) ) );
          self->setProperty( "lastError", tcps->lastError() );
          self->setProperty( "timedOut", Item( false ) );
+         
+         throw  new NetError( ErrorParam( FALSOCK_ERR_GENERIC, __LINE__ )
+            .desc( FAL_STR( sk_msg_generic ) )
+            .sysError( (uint32) tcps->lastError() ) );
       }
    }
    else {
+      if ( timeout > 0 ) vm->unidle();
       self->setProperty( "timedOut", Item( false ) );
       vm->regA().setBoolean( true );
    }
@@ -370,9 +373,8 @@ FALCON_FUNC  Socket_writeAvailable( ::Falcon::VMachine *vm )
    Item *to = vm->param( 0 );
    if ( to != 0 && ! to->isOrdinal() )
    {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "[N]" ) ) );
-      return;
+      throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "[N]" ) );
    }
 
    int64 timeout = to == 0 ? -1 : int64(to->forceNumeric() * 1000.0);
@@ -381,8 +383,11 @@ FALCON_FUNC  Socket_writeAvailable( ::Falcon::VMachine *vm )
    Sys::Socket *tcps = (Sys::Socket *) self->getUserData();
    int res;
 
+   if ( timeout > 0 ) vm->idle();
    if ( ( res = tcps->writeAvailable( (int32)timeout, &vm->systemData() ) ) <= 0 )
    {
+      if ( timeout > 0 ) vm->unidle();
+      
       // interrupted?
       if ( res == -2 )
       {
@@ -396,14 +401,15 @@ FALCON_FUNC  Socket_writeAvailable( ::Falcon::VMachine *vm )
       }
       else {
          // error
-         vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_GENERIC, __LINE__ )
-            .desc( FAL_STR( sk_msg_generic ) )
-            .sysError( (uint32) tcps->lastError() ) ) );
          self->setProperty( "lastError", tcps->lastError() );
          self->setProperty( "timedOut", Item( false ) );
+         throw  new NetError( ErrorParam( FALSOCK_ERR_GENERIC, __LINE__ )
+            .desc( FAL_STR( sk_msg_generic ) )
+            .sysError( (uint32) tcps->lastError() ) );
       }
    }
    else {
+      if ( timeout > 0 ) vm->unidle();
       self->setProperty( "timedOut", Item( false ) );
       vm->regA().setBoolean( true );
    }
@@ -422,7 +428,7 @@ FALCON_FUNC  Socket_getHost( ::Falcon::VMachine *vm )
 {
    CoreObject *self = vm->self().asObject();
    Sys::Socket *tcps = (Sys::Socket *) self->getUserData();
-   String *s = new GarbageString( vm );
+   String *s = new CoreString;
    if ( tcps->address().getAddress( *s ) ) {
       vm->retval( s );
       return;
@@ -442,14 +448,14 @@ FALCON_FUNC  Socket_getHost( ::Falcon::VMachine *vm )
    has been issued. Returned values are a system-specific 1:1 mapping of numeric
    ports to service names. If the port has not an associated service name,
    the port number is returned as a string value (I.e. port 80 is
-   returned as the string “80”).
+   returned as the string "80").
 */
 
 FALCON_FUNC  Socket_getService( ::Falcon::VMachine *vm )
 {
    CoreObject *self = vm->self().asObject();
    Sys::Socket *tcps = (Sys::Socket *) self->getUserData();
-   String *s = new GarbageString( vm );
+   String *s = new CoreString;
    if ( tcps->address().getService( *s ) ) {
       vm->retval( s );
       return;
@@ -509,9 +515,9 @@ FALCON_FUNC  TCPSocket_init( ::Falcon::VMachine *vm )
 
    if ( skt->lastError() != 0 ) {
       self->setProperty( "lastError", (int64) skt->lastError() );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_CREATE, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_CREATE, __LINE__ )
          .desc( FAL_STR( sk_msg_errcreate ) )
-         .sysError( (uint32) skt->lastError() ) ) );
+         .sysError( (uint32) skt->lastError() ) );
    }
 }
 
@@ -549,9 +555,8 @@ FALCON_FUNC  TCPSocket_connect( ::Falcon::VMachine *vm )
    if ( i_server == 0 || i_service == 0 || ! i_server->isString() ||
          ! i_service->isString() )
    {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S, S" ) ) );
-      return;
+      throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S, S" ) );
    }
 
    // try to resolve them.
@@ -561,18 +566,20 @@ FALCON_FUNC  TCPSocket_connect( ::Falcon::VMachine *vm )
    //in case of failed resolution, raise an error.
    if ( ! addr.resolve() ) {
       self->setProperty( "lastError", addr.lastError() );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_RESOLV, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_RESOLV, __LINE__ )
          .desc( FAL_STR( sk_msg_errcreate ) )
-         .sysError( (uint32) addr.lastError() ) ) );
-      return;
+         .sysError( (uint32) addr.lastError() ) );
    }
 
+   vm->idle();
    // connection
    if ( tcps->connect( addr ) ) {
+      vm->unidle();
       vm->regA().setBoolean( true );
       self->setProperty( "timedOut", Item( false ) );
       return;
    }
+   vm->unidle();
 
    // connection not complete
    if ( tcps->lastError() == 0 ) {
@@ -584,9 +591,9 @@ FALCON_FUNC  TCPSocket_connect( ::Falcon::VMachine *vm )
       self->setProperty( "lastError", tcps->lastError() );
       self->setProperty( "timedOut", Item( false ) );
 
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_CONNECT, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_CONNECT, __LINE__ )
          .desc( FAL_STR( sk_msg_errconnect ) )
-         .sysError( (uint32) tcps->lastError() ) ) );
+         .sysError( (uint32) tcps->lastError() ) );
    }
 }
 
@@ -614,9 +621,10 @@ FALCON_FUNC  TCPSocket_isConnected( ::Falcon::VMachine *vm )
 
       // an error!
       self->setProperty( "lastError", tcps->lastError() );
-      vm->raiseModError( new NetError( ErrorParam( FALSOCK_ERR_CONNECT, __LINE__ )
+      self->setProperty( "timedOut", Item( false ) );
+      throw new NetError( ErrorParam( FALSOCK_ERR_CONNECT, __LINE__ )
          .desc( FAL_STR( sk_msg_errconnect ) )
-         .sysError( (uint32) tcps->lastError() ) ) );
+         .sysError( (uint32) tcps->lastError() ) );
    }
    else {
       // success
@@ -642,9 +650,9 @@ FALCON_FUNC  TCPSocket_isConnected( ::Falcon::VMachine *vm )
    case the sent data may get corrupted as a transmission may deliver only part
    of a character or of a number stored in a memory buffer.
 
-   If a @a size parameter is not specified, the method will try to send the whole
+   If a @b size parameter is not specified, the method will try to send the whole
    content of the buffer, otherwise it will send at maximum size bytes. If a
-   @a start parameter is specified, then the data sent will be taken starting
+   @b start parameter is specified, then the data sent will be taken starting
    from that position in the buffer (counting in bytes from the start).
 
    This is useful when sending big buffers in several steps, so that
@@ -672,9 +680,8 @@ FALCON_FUNC  TCPSocket_send( ::Falcon::VMachine *vm )
        ( start != 0 && ! start->isOrdinal() )
       )
    {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S, [I], [I]" ) ) );
-      return;
+      throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S, [I], [I]" ) );
    }
 
    String *dataStr = data->asString();
@@ -686,13 +693,15 @@ FALCON_FUNC  TCPSocket_send( ::Falcon::VMachine *vm )
       count = dataStr->size() - start_pos;
    }
 
+   vm->idle();
    int32 res = tcps->send( dataStr->getRawStorage() + start_pos, count );
+   vm->unidle();
+   
    if( res == -1 ) {
       self->setProperty( "lastError", tcps->lastError() );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_SEND, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_SEND, __LINE__ )
          .desc( FAL_STR( sk_msg_errsend ) )
-         .sysError( (uint32) tcps->lastError() ) ) );
-      return;
+         .sysError( (uint32) tcps->lastError() ) );
    }
    else if ( res == -2 )
       self->setProperty( "timedOut", Item( true ) );
@@ -706,7 +715,7 @@ FALCON_FUNC  TCPSocket_send( ::Falcon::VMachine *vm )
    @method recv TCPSocket
    @brief Reads incoming data.
    @param bufOrSize A pre-allocated buffer to fill, or a maximum size in bytes to be read.
-   @optParam size Maximum size in bytes to be read.
+   @optparam size Maximum size in bytes to be read.
    @return If @b bufOrSize is a size, returns a filled string buffer, otherwise it returns
       the amount of bytes actually read.
    @raise NetError on network error.
@@ -753,17 +762,15 @@ FALCON_FUNC  TCPSocket_recv( ::Falcon::VMachine *vm )
    bool returnTarget;
 
    if ( target == 0 ) {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "X, [N]" ) ) );
-      return;
+      throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "X, [N]" ) );
    }
 
    if ( last != 0 ) {
       size = (int32) last->forceInteger();
       if ( size <= 0 ) {
-         vm->raiseModError(  new ParamError( ErrorParam( e_param_range, __LINE__ ).
-            extra( FAL_STR( sk_msg_lesszero) ) ) );
-         return;
+         throw  new ParamError( ErrorParam( e_param_range, __LINE__ ).
+            extra( FAL_STR( sk_msg_lesszero) ) );
       }
 
       if ( target->isString() )
@@ -773,9 +780,8 @@ FALCON_FUNC  TCPSocket_recv( ::Falcon::VMachine *vm )
          cs_target->reserve( size );
       }
       else {
-         vm->raiseModError(  new ParamError( ErrorParam( e_param_type, __LINE__ ).
-            extra( FAL_STR( sk_msg_firstnostring ) ) ) );
-         return;
+         throw  new ParamError( ErrorParam( e_param_type, __LINE__ ).
+            extra( FAL_STR( sk_msg_firstnostring ) ) );
       }
       returnTarget = false;
    }
@@ -789,9 +795,8 @@ FALCON_FUNC  TCPSocket_recv( ::Falcon::VMachine *vm )
       if ( size <= 0 ) {
          size = cs_target->size();
          if ( size <= 0 ) {
-            vm->raiseModError(  new ParamError( ErrorParam( e_param_range, __LINE__ ).
-               extra( FAL_STR( sk_msg_stringnospace ) ) ) );
-            return;
+            throw  new ParamError( ErrorParam( e_param_range, __LINE__ ).
+               extra( FAL_STR( sk_msg_stringnospace ) ) );
          }
 
          cs_target->reserve( size ); // force to bufferize
@@ -803,30 +808,29 @@ FALCON_FUNC  TCPSocket_recv( ::Falcon::VMachine *vm )
    {
       size = (int32) target->forceInteger();
       if ( size <= 0 ) {
-         vm->raiseModError(  new ParamError( ErrorParam( e_param_range, __LINE__ ).
-            extra( FAL_STR( sk_msg_lesszero) ) ) );
-         return;
+         throw  new ParamError( ErrorParam( e_param_range, __LINE__ ).
+            extra( FAL_STR( sk_msg_lesszero) ) );
       }
-      cs_target = new GarbageString( vm );
+      cs_target = new CoreString;
       cs_target->reserve( size );
       // no need to store for garbage, as we'll return this.
       returnTarget = true;
    }
    else
    {
-      vm->raiseModError(  new ParamError( ErrorParam( e_param_type, __LINE__ ).
-         extra( "X, [S|I]" ) ) );
-      return;
+      throw new ParamError( ErrorParam( e_param_type, __LINE__ ).
+         extra( "X, [S|I]" ) );
    }
 
-
+   vm->idle();
    size = tcps->recv( cs_target->getRawStorage(), size );
+   vm->unidle();
+   
    if( size == -1 ) {
       self->setProperty( "lastError", tcps->lastError() ) ;
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_RECV, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_RECV, __LINE__ )
          .desc( FAL_STR( sk_msg_errrecv ) )
-         .sysError( (uint32) tcps->lastError() ) ) );
-      return;
+         .sysError( (uint32) tcps->lastError() ) );
    }
    else if ( size == -2 ) {
       self->setProperty( "timedOut", Item( true ) );
@@ -871,8 +875,10 @@ FALCON_FUNC  TCPSocket_closeRead( ::Falcon::VMachine *vm )
 {
    CoreObject *self = vm->self().asObject();
    Sys::TCPSocket *tcps = (Sys::TCPSocket *) self->getUserData();
-
+   
+   vm->idle();
    if ( ! tcps->closeRead() ) {
+      vm->unidle();
       // may time out
       if ( tcps->lastError() == 0 ) {
          self->setProperty( "timedOut", Item( true ) );
@@ -883,13 +889,14 @@ FALCON_FUNC  TCPSocket_closeRead( ::Falcon::VMachine *vm )
       // an error!
       self->setProperty( "lastError", tcps->lastError() );
       self->setProperty( "timedOut", Item( false ) );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_CLOSE, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_CLOSE, __LINE__ )
          .desc( FAL_STR( sk_msg_errclose ) )
-         .sysError( (uint32) tcps->lastError() ) ) );
-      return;
+         .sysError( (uint32) tcps->lastError() ) );
+   }
+   else {
+      vm->unidle();
+      vm->regA().setBoolean( true );
    }
-
-   vm->regA().setBoolean( true );
 }
 
 
@@ -919,16 +926,19 @@ FALCON_FUNC  TCPSocket_closeWrite( ::Falcon::VMachine *vm )
    Sys::TCPSocket *tcps = (Sys::TCPSocket *) self->getUserData();
 
    self->setProperty( "timedOut", Item( false ) );
-
+   
+   vm->idle();
    if ( tcps->closeWrite() ) {
+      vm->unidle();
       vm->regA().setBoolean( true );
    }
    else {
       // an error!
+      vm->unidle();
       self->setProperty( "lastError", tcps->lastError() );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_CLOSE, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_CLOSE, __LINE__ )
          .desc( FAL_STR( sk_msg_errclose ) )
-         .sysError( (uint32) tcps->lastError() ) ) );
+         .sysError( (uint32) tcps->lastError() ) );
    }
 }
 
@@ -953,7 +963,10 @@ FALCON_FUNC  TCPSocket_close( ::Falcon::VMachine *vm )
    CoreObject *self = vm->self().asObject();
    Sys::TCPSocket *tcps = (Sys::TCPSocket *) self->getUserData();
 
-   if ( ! tcps->close() ) {
+   vm->idle();
+   if ( ! tcps->close() ) 
+   {
+      vm->unidle();
       // may time out
       if ( tcps->lastError() == 0 ) {
          self->setProperty( "timedOut", Item( true ) );
@@ -964,13 +977,14 @@ FALCON_FUNC  TCPSocket_close( ::Falcon::VMachine *vm )
       // an error!
       self->setProperty( "lastError", tcps->lastError() );
       self->setProperty( "timedOut", Item( false ) );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_CLOSE, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_CLOSE, __LINE__ )
          .desc( FAL_STR( sk_msg_errclose ) )
-         .sysError( (uint32) tcps->lastError() ) ) );
-      return;
+         .sysError( (uint32) tcps->lastError() ) );
+   }
+   else {
+      vm->unidle();
+      vm->regA().setBoolean( true );
    }
-
-   vm->regA().setBoolean( true );
 }
 
 // ==============================================
@@ -1004,8 +1018,8 @@ FALCON_FUNC  UDPSocket_init( ::Falcon::VMachine *vm )
    {
       if ( ! address_i->isString() || ( service_i != 0 && ! service_i->isString() ) )
       {
-         vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-            extra( "S, [S]" ) ) );
+         throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+            extra( "S, [S]" ) );
          return;
       }
       Sys::Address addr;
@@ -1024,15 +1038,16 @@ FALCON_FUNC  UDPSocket_init( ::Falcon::VMachine *vm )
 
    if ( skt->lastError() != 0 ) {
       self->setProperty( "lastError", (int64) skt->lastError() );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_CREATE, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_CREATE, __LINE__ )
          .desc( FAL_STR( sk_msg_errcreate ) )
-         .sysError( (uint32) skt->lastError() ) ) );
+         .sysError( (uint32) skt->lastError() ) );
    }
 }
 
 
 /*#
    @method sendTo UDPSocket
+   @brief Sends a datagram to a given address.
    @param host Remote host where to send the datagram.
    @param service Remote service or port number where to send the datagram.
    @param buffer The buffer to be sent.
@@ -1046,11 +1061,11 @@ FALCON_FUNC  UDPSocket_init( ::Falcon::VMachine *vm )
    filled before being sent, provided that the specified size
    does not exceed datagram size limits.
 
-   The @a host parameter may be an host name to be resolved or an address;
+   The @b host parameter may be an host name to be resolved or an address;
    if the @a UDPSocket.broadcast method has been successfully called,
    it may be also a multicast or broadcast address.
 
-   The @a service parameter is a string containing either a service name
+   The @b service parameter is a string containing either a service name
    (i.e. "http") or  a numeric port number (i.e. "80", as a string).
 
    The @b buffer may be a byte-only string or a
@@ -1093,8 +1108,8 @@ FALCON_FUNC  UDPSocket_sendTo( ::Falcon::VMachine *vm )
        ( start != 0 && ! start->isOrdinal() )
       )
    {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "S, S, [N], [N]" ) ) );
+      throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "S, S, [N], [N]" ) );
       return;
    }
 
@@ -1107,14 +1122,19 @@ FALCON_FUNC  UDPSocket_sendTo( ::Falcon::VMachine *vm )
       count = dataStr->size() - start_pos;
    }
 
+
    Sys::Address target;
    target.set( *addr->asString(), *srvc->asString() );
+   
+   vm->idle();
    int32 res = udps->sendTo( dataStr->getRawStorage() + start_pos, count, target );
+   vm->unidle();
+   
    if( res == -1 ) {
       self->setProperty( "lastError", udps->lastError() );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_SEND, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_SEND, __LINE__ )
          .desc( FAL_STR( sk_msg_errsend ) )
-         .sysError( (uint32) udps->lastError() ) ) );
+         .sysError( (uint32) udps->lastError() ) );
       return;
    }
    else if ( res == -2 )
@@ -1130,7 +1150,7 @@ FALCON_FUNC  UDPSocket_sendTo( ::Falcon::VMachine *vm )
    @method recv UDPSocket
    @brief Reads incoming data.
    @param bufOrSize A pre-allocated buffer to fill, or a maximum size in bytes to be read.
-   @optParam size Maximum size in bytes to be read.
+   @optparam size Maximum size in bytes to be read.
    @return If @b bufOrSize is a size, returns a filled string buffer, otherwise it returns
       the amount of bytes actually read.
    @raise NetError on network error.
@@ -1184,16 +1204,15 @@ FALCON_FUNC  UDPSocket_recv( ::Falcon::VMachine *vm )
    bool returnTarget;
 
    if ( target == 0 ) {
-      vm->raiseModError(  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "X, [N]" ) ) );
-      return;
+      throw  new ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "X, [N]" ) );
    }
 
    if ( last != 0 ) {
       size = (int32) last->forceInteger();
       if ( size <= 0 ) {
-         vm->raiseModError(  new ParamError( ErrorParam( e_param_range, __LINE__ ).
-            extra( FAL_STR( sk_msg_lesszero) ) ) );
+         throw  new ParamError( ErrorParam( e_param_range, __LINE__ ).
+            extra( FAL_STR( sk_msg_lesszero) ) );
          return;
       }
 
@@ -1204,9 +1223,8 @@ FALCON_FUNC  UDPSocket_recv( ::Falcon::VMachine *vm )
          cs_target->reserve( size );
       }
       else {
-         vm->raiseModError(  new ParamError( ErrorParam( e_param_type, __LINE__ ).
-            extra( FAL_STR( sk_msg_firstnostring ) ) ) );
-            return;
+         throw  new ParamError( ErrorParam( e_param_type, __LINE__ ).
+            extra( FAL_STR( sk_msg_firstnostring ) ) );
       }
       returnTarget = false;
    }
@@ -1218,9 +1236,8 @@ FALCON_FUNC  UDPSocket_recv( ::Falcon::VMachine *vm )
       size = cs_target->size();
 
       if ( size <= 0 ) {
-         vm->raiseModError(  new ParamError( ErrorParam( e_param_range, __LINE__ ).
-            extra( FAL_STR( sk_msg_stringnospace ) ) ) );
-         return;
+         throw  new ParamError( ErrorParam( e_param_range, __LINE__ ).
+            extra( FAL_STR( sk_msg_stringnospace ) ) );
       }
 
       cs_target->reserve( size ); // force to bufferize
@@ -1230,30 +1247,30 @@ FALCON_FUNC  UDPSocket_recv( ::Falcon::VMachine *vm )
    {
       size = (int32) target->forceInteger();
       if ( size <= 0 ) {
-         vm->raiseModError(  new ParamError( ErrorParam( e_param_range, __LINE__ ).
-            extra( FAL_STR( sk_msg_lesszero) ) ) );
-         return;
+         throw  new ParamError( ErrorParam( e_param_range, __LINE__ ).
+            extra( FAL_STR( sk_msg_lesszero) ) );
       }
-      cs_target = new GarbageString( vm );
+      cs_target = new CoreString;
       cs_target->reserve( size );
       // no need to store for garbage, as we'll return this.
       returnTarget = true;
    }
    else
    {
-      vm->raiseModError( new  ParamError( ErrorParam( e_param_type, __LINE__ ).
-         extra( "X, S|I" ) ) );
-      return;
+      throw new  ParamError( ErrorParam( e_param_type, __LINE__ ).
+         extra( "X, S|I" ) );
    }
 
    Sys::Address from;
+   vm->idle();
    size = udps->recvFrom( cs_target->getRawStorage(), size, from );
+   vm->unidle();
+
    if( size == -1 ) {
       self->setProperty( "lastError", udps->lastError() ) ;
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_SEND, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_SEND, __LINE__ )
          .desc( FAL_STR( sk_msg_errsend ) )
-         .sysError( (uint32) udps->lastError() ) ) );
-      return;
+         .sysError( (uint32) udps->lastError() ) );
    }
    else if ( size == -2 ) {
       self->setProperty( "timedOut", Item( true ) );
@@ -1334,9 +1351,9 @@ FALCON_FUNC  TCPServer_init( ::Falcon::VMachine *vm )
 
    if ( skt->lastError() != 0 ) {
       self->setProperty( "lastError", (int64) skt->lastError() );
-      vm->raiseModError(  new NetError( ErrorParam( FALSOCK_ERR_CREATE, __LINE__ )
+      throw  new NetError( ErrorParam( FALSOCK_ERR_CREATE, __LINE__ )
          .desc( FAL_STR( sk_msg_errcreate ) )
-         .sysError( (uint32) skt->lastError() ) ) );
+         .sysError( (uint32) skt->lastError() ) );
    }
 }
 
@@ -1390,8 +1407,8 @@ FALCON_FUNC  TCPServer_bind( ::Falcon::VMachine *vm )
 
    if ( i_first == 0 || ! i_first->isString() || ( i_second != 0 && ! i_second->isString() ) )
    {
-         vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-            extra( "S, [S]" ) ) );
+         throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+            extra( "S, [S]" ) );
       return;
    }
 
@@ -1404,9 +1421,9 @@ FALCON_FUNC  TCPServer_bind( ::Falcon::VMachine *vm )
    if ( ! srvs->bind( addr ) )
    {
       self->setProperty( "lastError", srvs->lastError() );
-      vm->raiseModError(  new  NetError( ErrorParam( FALSOCK_ERR_BIND, __LINE__ )
+      throw  new  NetError( ErrorParam( FALSOCK_ERR_BIND, __LINE__ )
          .desc( FAL_STR(sk_msg_errbind) )
-         .sysError( (uint32) srvs->lastError() ) ) );
+         .sysError( (uint32) srvs->lastError() ) );
    }
 
    vm->retnil();
@@ -1448,17 +1465,20 @@ FALCON_FUNC  TCPServer_accept( ::Falcon::VMachine *vm )
       srvs->timeout( (int32) to->forceInteger() );
    }
    else {
-      vm->raiseModError( new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
-         extra( "[N]" ) ) );
+      throw new  ParamError( ErrorParam( e_inv_params, __LINE__ ).
+         extra( "[N]" ) );
       return;
    }
 
+   vm->idle();
    Sys::TCPSocket *skt = srvs->accept();
+   vm->unidle();
+
    if ( srvs->lastError() != 0 )
    {
       self->setProperty( "lastError", srvs->lastError() );
-      vm->raiseModError( new  NetError( ErrorParam( FALSOCK_ERR_ACCEPT, __LINE__ ).
-         desc( FAL_STR( sk_msg_erraccept ) ).sysError( (uint32) srvs->lastError() ) ) );
+      throw new  NetError( ErrorParam( FALSOCK_ERR_ACCEPT, __LINE__ ).
+         desc( FAL_STR( sk_msg_erraccept ) ).sysError( (uint32) srvs->lastError() ) );
       return;
    }
 
