@@ -22,7 +22,7 @@
 #define FLC_SOCKET_SYS_H
 
 #include <falcon/string.h>
-#include <falcon/userdata.h>
+#include <falcon/falcondata.h>
 #include <falcon/vm_sys.h>
 
 namespace Falcon {
@@ -62,9 +62,9 @@ public:
 
    Address():
       m_systemData(0),
+      m_port(0),
       m_resolvCount(0),
-      m_activeHostId(-1),
-      m_port(0)
+      m_activeHostId(-1)
       {}
 
    ~Address();
@@ -141,7 +141,7 @@ public:
 };
 
 /** Base class for system dependant socket implementation. */
-class Socket: public Falcon::UserData
+class Socket: public Falcon::FalconData
 {
 protected:
 
@@ -165,10 +165,10 @@ protected:
    }
 
    Socket( void *systemData, bool ipv6 = false ):
+      m_ipv6(ipv6 ),
       m_lastError(0),
       m_timeout(0),
-      m_boundFamily(0),
-      m_ipv6(ipv6 )
+      m_boundFamily(0)
    {
       d.m_systemData = systemData;
    }
@@ -195,6 +195,9 @@ public:
       Clears the socket internal data without disconnecting.
    */
    void terminate();
+
+   virtual void gcMark( uint32 mk ) {};
+   virtual FalconData *clone() const { return 0; }
 };
 
 class UDPSocket: public Socket
